/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _BT_REQUEST_SENDER_H_
#define _BT_REQUEST_SENDER_H_

#include <sys/types.h>
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	int service_function;
	void *cb;
	void *user_data;
} bt_req_info_t;

int _bt_sync_send_request(int service_type, int service_function,
			GArray *in_param1, GArray *in_param2,
			GArray *in_param3, GArray *in_param4,
			GArray **out_param1);

/* The caller of this should pass #define macro/enum types as the first two params */
#define _bt_send_request(a, b, format ...) ( \
	{ \
	if ((a != BT_CHECK_PRIVILEGE && (a == BT_BLUEZ_SERVICE || a == BT_OBEX_SERVICE)) && \
		(b != BT_GET_LOCAL_NAME && b != BT_GET_LOCAL_ADDRESS && \
		 b != BT_GET_BONDED_DEVICE && b != BT_IS_DEVICE_CONNECTED && \
		 b != BT_GET_BONDED_DEVICES && b != BT_GET_DISCOVERABLE_MODE)) \
		BT_INFO_C("Sync Request => type=%s, fn=%s(0x%x)", #a, #b, b); \
	else \
		BT_DBG("Sync Request => type=%s, fn=%s(0x%x)", #a, #b, b); \
	_bt_sync_send_request(a, b, format); \
	} \
	)

int _bt_async_send_request(int service_type, int service_function,
			GArray *in_param1, GArray *in_param2,
			GArray *in_param3, GArray *in_param4,
			void *callback, void *user_data);

/* The caller of this should pass #define macro/enum types as the first two params */
#define _bt_send_request_async(a, b, format ...) ( \
	{ \
	if ((a != BT_CHECK_PRIVILEGE && (a == BT_BLUEZ_SERVICE || a == BT_OBEX_SERVICE)) && \
		(b != BT_GET_LOCAL_NAME && b != BT_GET_LOCAL_ADDRESS && \
		 b != BT_GET_BONDED_DEVICE && b != BT_IS_DEVICE_CONNECTED && \
		 b != BT_GET_BONDED_DEVICES && b != BT_GET_DISCOVERABLE_MODE)) \
		BT_INFO_C("Async Request => type=%s, fn=%s(0x%x)", #a, #b, b); \
	else \
		BT_DBG("Async Request => type=%s, fn=%s(0x%x)", #a, #b, b); \
	_bt_async_send_request(a, b, format); \
	} \
	)

void _bt_gdbus_deinit_proxys(void);

int _bt_sync_send_request_with_unix_fd_list(
		int service_type, int service_function,
		GArray *in_param1, GArray *in_param2,
		GArray *in_param3, GArray *in_param4,
		GUnixFDList *fd_list, GArray **out_param1,
		GUnixFDList **out_fd_list);
#define _bt_send_request_with_unix_fd_list(a, b, format ...) ( \
	{ \
	BT_INFO_C("Async Request => type=%s, fn=%s(0x%x)", #a, #b, b); \
	_bt_sync_send_request_with_unix_fd_list(a, b, format); \
	} \
	)

int _bt_async_send_request_with_unix_fd_list(int service_type, int service_function,
			GArray *in_param1, GArray *in_param2,
			GArray *in_param3, GArray *in_param4,
			void *callback, void *user_data,
			GUnixFDList *fd_list, GAsyncReadyCallback __async_req_cb);
#define _bt_send_request_async_with_unix_fd_list(a, b, format ...) ( \
	{ \
	BT_INFO_C("Async Request => type=%s, fn=%s(0x%x)", #a, #b, b); \
	_bt_async_send_request_with_unix_fd_list(a, b, format); \
	} \
	)

#ifdef __cplusplus
}
#endif

#endif /* _BT_REQUEST_SENDER_H_ */
