/*
 * Copyright (c) 2015 -2016 Samsung Electronics Co., Ltd All Rights Reserved.
 *
 * Contact: Anupam Roy <anupam.r@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef _BT_HAL_MSG_H_
#define _BT_HAL_MSG_H_

#define HAL_MINIMUM_EVENT		0x81
/*TODO: More events to be added in subsequent patches */

/* HAL Global Macros */
#define HAL_POWER_OFF                   0x00
#define HAL_POWER_ON                    0x01

#define HAL_PROP_ADAPTER_NAME                   0x01
#define HAL_PROP_ADAPTER_ADDR                   0x02
#define HAL_PROP_ADAPTER_UUIDS                  0x03
#define HAL_PROP_ADAPTER_CLASS                  0x04
#define HAL_PROP_ADAPTER_TYPE                   0x05
#define HAL_PROP_ADAPTER_SERVICE_REC            0x06
#define HAL_PROP_ADAPTER_SCAN_MODE              0x07
#define HAL_PROP_ADAPTER_BONDED_DEVICES         0x08
#define HAL_PROP_ADAPTER_DISC_TIMEOUT           0x09


#define HAL_PROP_DEVICE_NAME                    0x01
#define HAL_PROP_DEVICE_ADDR                    0x02
#define HAL_PROP_DEVICE_UUIDS                   0x03
#define HAL_PROP_DEVICE_CLASS                   0x04
#define HAL_PROP_DEVICE_TYPE                    0x05
#define HAL_PROP_DEVICE_SERVICE_REC             0x06

/* Tizen specific HAL Adapter and Device property types.
   These properties have to added to HAL bt_property_type_t enums */

struct hal_prop_device_service_rec {
	uint8_t uuid[16];
	uint16_t channel;
	uint8_t name_len;
	uint8_t name[];
} __attribute__((packed));

#define HAL_PROP_DEVICE_FRIENDLY_NAME            0x0a
#define HAL_PROP_DEVICE_RSSI                     0x0b
#define HAL_PROP_DEVICE_VERSION_INFO             0x0c
/*
 * Tizen specific HAL Adapter and Device property types.
 * These properties have to added to HAL bt_property_type_t enums
 */
#define HAL_PROP_DEVICE_PAIRED                  0x0d
#define HAL_PROP_DEVICE_CONNECTED               0x0e
#define HAL_PROP_DEVICE_TRUSTED                 0x0f
#define HAL_PROP_ADAPTER_PAIRABLE               0x10
#define HAL_PROP_ADAPTER_PAIRABLE_TIMEOUT       0x11
#define HAL_PROP_ADAPTER_VERSION		0x12
#define HAL_PROP_ADAPTER_IPSP_INITIALIZED	0x13
#define HAL_PROP_ADAPTER_MODALIAS		0x14

#define HAL_PROP_DEVICE_MANUFACTURER_DATA_LEN   0x15
#define HAL_PROP_DEVICE_MANUFACTURER_DATA       0x16
#define HAL_PROP_DEVICE_BLE_ADV_DATA            0x17
#define HAL_PROP_ADAPTER_LOCAL_LE_FEAT          0x18
#define HAL_PROP_DEVICE_IS_ALIAS_SET		0x19

struct hal_prop_device_info {
	uint8_t version;
	uint16_t sub_version;
	uint16_t manufacturer;
} __attribute__((packed));

#define HAL_PROP_DEVICE_TIMESTAMP               0xFF

#define HAL_EV_ADAPTER_STATE_CHANGED    0x00
struct hal_ev_adapter_state_changed {
	uint8_t state;
} __attribute__((packed));


struct hal_property {
	uint8_t  type;
	uint16_t len;
	uint8_t  val[0];
} __attribute__((packed));

#define HAL_EV_ADAPTER_PROPS_CHANGED    0x01
struct hal_ev_adapter_props_changed {
	uint8_t              status;
	uint8_t              num_props;
	struct  hal_property props[0];
} __attribute__((packed));


#define HAL_DISCOVERY_STATE_STOPPED     0x00
#define HAL_DISCOVERY_STATE_STARTED     0x01

#define HAL_EV_DISCOVERY_STATE_CHANGED  0x85
struct hal_ev_discovery_state_changed {
	uint8_t state;
} __attribute__((packed));

#define HAL_EV_REMOTE_DEVICE_PROPS      0x83
struct hal_ev_remote_device_props {
	uint8_t             status;
	uint8_t             bdaddr[6];
	uint8_t             num_props;
	struct hal_property props[0];
} __attribute__((packed));

#define HAL_EV_DEVICE_FOUND             0x84
struct hal_ev_device_found {
	uint8_t             num_props;
	struct hal_property props[0];
} __attribute__((packed));


/* Device callbacks */
#define HAL_EV_PIN_REQUEST              0x86
struct hal_ev_pin_request {
	uint8_t  bdaddr[6];
	uint8_t  name[249];
	uint32_t class_of_dev;
} __attribute__((packed));

#define HAL_EV_SSP_REQUEST              0x87
struct hal_ev_ssp_request {
	uint8_t  bdaddr[6];
	uint8_t  name[249];
	uint32_t class_of_dev;
	uint8_t  pairing_variant;
	uint32_t passkey;
} __attribute__((packed));

#define HAL_BOND_STATE_NONE 0
#define HAL_BOND_STATE_BONDING 1
#define HAL_BOND_STATE_BONDED 2

#define HAL_EV_BOND_STATE_CHANGED       0x88
struct hal_ev_bond_state_changed {
	uint8_t status;
	uint8_t bdaddr[6];
	uint8_t state;
} __attribute__((packed));

#define HAL_EV_AUTHORIZE_REQUEST        0x89
struct hal_ev_authorize_request {
	uint8_t  bdaddr[6];
	uint32_t service_id;
} __attribute__((packed));

#ifdef TIZEN_BT_HAL
#define HAL_EV_SOCK_AUTHORIZE_REQUEST        0x90
struct hal_ev_sock_conn_auth {
	uint8_t uuid[16];
	uint8_t bdaddr[6];
} __attribute__((packed));
#endif //TIZEN_BT_HAL

#define HAL_ACL_STATE_CONNECTED         0x00
#define HAL_ACL_STATE_DISCONNECTED      0x01

#define HAL_EV_ACL_STATE_CHANGED        0x8A
struct hal_ev_acl_state_changed {
	uint8_t status;
	uint8_t bdaddr[6];
	uint8_t state;
} __attribute__((packed));

#define HAL_DEVICE_TRUSTED         0x00
#define HAL_DEVICE_UNTRUSTED       0x01

#define HAL_EV_DEVICE_TRUST_CHANGED	0x8B
struct hal_ev_device_trust_state_changed {
	uint8_t bdaddr[6];
	uint8_t trust;
} __attribute__((packed));

#define BT_TRANSPORT_UNKNOWN            0x00
#define BT_TRANSPORT_BR_EDR             0x01
#define BT_TRANSPORT_LE                 0x02

/* HID host events */
#define HAL_HIDHOST_STATE_CONNECTED	0x00
#define HAL_HIDHOST_STATE_CONNECTING	0x01
#define HAL_HIDHOST_STATE_DISCONNECTED	0x02
#define HAL_HIDHOST_STATE_DISCONNECTING	0x03
#define HAL_HIDHOST_STATE_NO_HID	0x07
#define HAL_HIDHOST_STATE_FAILED	0x08
#define HAL_HIDHOST_STATE_UNKNOWN	0x09

#define HAL_EV_HIDHOST_CONN_STATE		0x81
struct hal_ev_hidhost_conn_state {
	uint8_t bdaddr[6];
	uint8_t state;
} __attribute__((packed));

#define HAL_EV_HIDHOST_INFO	0x82
struct hal_ev_hidhost_info {
	uint8_t  bdaddr[6];
	uint8_t  attr;
	uint8_t  subclass;
	uint8_t  app_id;
	uint16_t vendor;
	uint16_t product;
	uint16_t version;
	uint8_t  country;
	uint16_t descr_len;
	uint8_t  descr[884];
} __attribute__((packed));

#define HAL_EV_HIDHOST_PROTO_MODE	0x83
struct hal_ev_hidhost_proto_mode {
	uint8_t bdaddr[6];
	uint8_t status;
	uint8_t mode;
} __attribute__((packed));

#define HAL_EV_HIDHOST_IDLE_TIME	0x84
struct hal_ev_hidhost_idle_time {
	uint8_t bdaddr[6];
	uint8_t status;
	uint32_t idle_rate;
} __attribute__((packed));

#define HAL_EV_HIDHOST_GET_REPORT	0x85
struct hal_ev_hidhost_get_report {
	uint8_t  bdaddr[6];
	uint8_t  status;
	uint16_t len;
	uint8_t  data[0];
} __attribute__((packed));

#define HAL_EV_HIDHOST_VIRTUAL_UNPLUG	0x86
struct hal_ev_hidhost_virtual_unplug {
	uint8_t  bdaddr[6];
	uint8_t  status;
} __attribute__((packed));

#define HAL_EV_HIDHOST_HANDSHAKE	0x87
struct hal_ev_hidhost_handshake {
	uint8_t  bdaddr[6];
	uint8_t  status;
} __attribute__((packed));

/* Bluetooth Socket HAL events */
struct hal_ev_sock_connect {
	short   size;
	uint8_t bdaddr[6];
	int     channel;
	int     status;
} __attribute__((packed));

/* A2DP (AV) Headers */
#define HAL_EV_A2DP_STATE_DISCONNECTED             0x00
#define HAL_EV_A2DP_STATE_CONNECTING               0x01
#define HAL_EV_A2DP_STATE_CONNECTED                0x02
#define HAL_EV_A2DP_STATE_DISCONNECTING            0x03

#define HAL_EV_A2DP_CONN_STATE                  0x8C
struct hal_ev_a2dp_conn_state {
	uint8_t state;
	uint8_t bdaddr[6];
} __attribute__((packed));

#define HAL_AUDIO_SUSPEND                       0x00
#define HAL_AUDIO_STOPPED                       0x01
#define HAL_AUDIO_STARTED                       0x02

#define HAL_EV_A2DP_AUDIO_STATE                 0x8D
struct hal_ev_a2dp_audio_state {
	uint8_t state;
	uint8_t bdaddr[6];
} __attribute__((packed));

#define HAL_EV_A2DP_AUDIO_CONFIG                0x8E
struct hal_ev_a2dp_audio_config {
	uint8_t  bdaddr[6];
	uint32_t sample_rate;
	uint8_t  channel_count;
} __attribute__((packed));

/* A2DP Sink Role connection*/
#define HAL_EV_A2DP_SOURCE_CONN_STATE                   0x8F

/* HandsFree Headers */
#define HAL_EV_HANDSFREE_CONN_STATE_DISCONNECTED        0x00
#define HAL_EV_HANDSFREE_CONN_STATE_CONNECTING          0x01
#define HAL_EV_HANDSFREE_CONN_STATE_CONNECTED           0x02
#define HAL_EV_HANDSFREE_CONN_STATE_SLC_CONNECTED       0x03
#define HAL_EV_HANDSFREE_CONN_STATE_DISCONNECTING       0x04

#define HAL_EV_HANDSFREE_CONN_STATE             0x91
struct hal_ev_handsfree_conn_state {
	uint8_t state;
	uint8_t bdaddr[6];
} __attribute__((packed));

#define HAL_EV_HANDSFREE_AUDIO_STATE_DISCONNECTED       0x00
#define HAL_EV_HANDSFREE_AUDIO_STATE_CONNECTING         0x01
#define HAL_EV_HANDSFREE_AUDIO_STATE_CONNECTED          0x02
#define HAL_EV_HANDSFREE_AUDIO_STATE_DISCONNECTING      0x04

#define HAL_EV_HANDSFREE_AUDIO_STATE            0x92
struct hal_ev_handsfree_audio_state {
	uint8_t state;
	uint8_t bdaddr[6];
} __attribute__((packed));

/* HDP HAL Events */
#define HAL_EV_HDP_APP_REG_STATE            0x93
struct hal_ev_hdp_app_reg_state {
	uint16_t app_id;
	uint8_t state;
} __attribute__((packed));

#define HAL_EV_HDP_CONN_STATE            0x94
struct hal_ev_hdp_conn_state {
	uint16_t app_id;
	uint8_t bdaddr[6];
	uint8_t mdep_index;
	uint16_t channel_id;
	uint8_t channel_state;
	int32_t data_fd;
} __attribute__((packed));

/* AVRCP Controller Role HAL Events */
#define HAL_EV_AVRCP_CTRL_CONN_STATE		0x95
#define HAL_AVRCP_CTRL_STATE_DISCONNECTED 0x00
#define HAL_AVRCP_CTRL_STATE_CONNECTED 0x01
struct hal_ev_avrcp_ctrl_conn_state {
	uint8_t state;
	uint8_t bdaddr[6];
} __attribute__((packed));

#define HAL_EV_AVRCP_CTRL_PASS_THROUGH_RSP	0x96
struct hal_ev_pass_cmd_rsp {
	uint8_t bdaddr[6];
	int key_code;
	int key_state;
} __attribute__((packed));

#define HAL_EV_AVRCP_CTRL_SET_PLAYER_APP_SETTING_RSP	0x97
#define HAL_AVRCP_PLAYER_APP_SETTING_REJECTED 0x00
#define HAL_AVRCP_PLAYER_APP_SETTING_ACCEPTED 0x01
struct hal_ev_set_player_app_setting_rsp {
	uint8_t bdaddr[6];
	uint8_t resp;
} __attribute__((packed));

#define HAL_EV_AVRCP_CTRL_PLAYER_APP_SETTING_CHANGED	0x98
#define HAL_MAX_APP_SETTINGS 8
#define HAL_PLAYER_ATTR_EQUALIZER 0x01
#define HAL_PLAYER_ATTR_REPEAT 0x02
#define HAL_PLAYER_ATTR_SHUFFLE 0x03
#define HAL_PLAYER_ATTR_SCAN 0x04
struct hal_ev_player_setting {
	uint8_t bdaddr[6];
	uint8_t num_attr;
	uint8_t attr_ids[HAL_MAX_APP_SETTINGS];
	uint8_t attr_values[HAL_MAX_APP_SETTINGS];
} __attribute__((packed));

#define HAL_EV_AVRCP_CTRL_TRACK_CHANGED		0x99
#define HAL_MAX_ATTR_STR_LEN 100
#define HAL_MAX_ATTR_NUM 10
#define HAL_MEDIA_ATTR_ID_INVALID -1
#define HAL_MEDIA_ATTR_ID_TITLE 0x00000001
#define HAL_MEDIA_ATTR_ID_ARTIST 0x00000002
#define HAL_MEDIA_ATTR_ID_ALBUM 0x00000003
#define HAL_MEDIA_ATTR_ID_TRACK_NUM 0x00000004
#define HAL_MEDIA_ATTR_ID_NUM_TRACKS 0x00000005
#define HAL_MEDIA_ATTR_ID_GENRE 0x00000006
#define HAL_MEDIA_ATTR_ID_PLAYING_TIME 0x00000007
struct hal_attr_val {
	uint32_t attr_id;
	uint8_t text[HAL_MAX_ATTR_STR_LEN];
} __attribute__((packed));

struct hal_ev_track_changed {
	uint8_t bdaddr[6];
	uint8_t num_attr;
	struct hal_attr_val attr[HAL_MAX_ATTR_NUM];
} __attribute__((packed));

#define HAL_EV_AVRCP_CTRL_PLAY_POSITION_CHANGED		0xA0
struct hal_ev_play_position {
	uint8_t bdaddr[6];
	uint32_t len;
	uint32_t pos;
} __attribute__((packed));

#define HAL_EV_AVRCP_CTRL_PLAY_STATUS_CHANGED		0xA1
#define HAL_PLAYSTATE_STOPPED 0x00    /* Stopped */
#define HAL_PLAYSTATE_PLAYING 0x01    /* Playing */
#define HAL_PLAYSTATE_PAUSED 0x02    /* Paused  */
#define HAL_PLAYSTATE_FWD_SEEK 0x03    /* Fwd Seek*/
#define HAL_PLAYSTATE_REV_SEEK 0x04    /* Rev Seek*/
#define HAL_PLAYSTATE_ERROR 0xFF    /* Error   */
struct hal_ev_play_status_changed {
	uint8_t bdaddr[6];
	uint8_t status;
} __attribute__((packed));

/* AVRCP TG events */
#ifdef TIZEN_BT_HAL
#define HAL_EV_AVRCP_TG_CONN_STATE		0xA2
#define HAL_EV_AVRCP_TG_DELAY_CHANGE	0xF1

#define HAL_AVRCP_TG_STATE_DISCONNECTED	0x00
#define HAL_AVRCP_TG_STATE_CONNECTED	0x01
struct hal_ev_avrcp_tg_conn_state{
	uint8_t bdaddr[6];
	uint8_t state;
} __attribute__((packed));

struct hal_ev_avrcp_tg_delay_changed{
	uint8_t bdaddr[6];
	uint16_t value;
} __attribute__((packed));

#endif /* TIZEN_BT_HAL */

#define HAL_EV_AVRCP_TG_SET_PLAYER_PROPERTY              0xA3

#define HAL_AVRCP_TG_PLAYER_PROP_EQUALIZER	0x00
#define HAL_AVRCP_TG_PLAYER_PROP_REPEAT		0x01
#define HAL_AVRCP_TG_PLAYER_PROP_SHUFFLE	0x02
#define HAL_AVRCP_TG_PLAYER_PROP_SCAN		0x03
struct hal_ev_avrcp_tg_player_property {
	uint8_t prop_type;
	uint8_t value;
} __attribute__((packed));

/* LE-GATT */
#define HAL_EV_SERVER_INSTANCE_INITIALIZED	0xA4
struct hal_ev_server_instance_registered {
	uint32_t status;
	uint32_t server_instance;
	uint8_t app_uuid[16];
} __attribute__((packed));

#define HAL_EV_MULTI_ADV_DATA_SET		0xA5
struct hal_ev_multi_adv_data_set {
	uint8_t status;
	uint8_t server_instance;
} __attribute__((packed));

#define HAL_EV_MULTI_ADV_ENABLE			0xA6
struct hal_ev_multi_adv_enable {
	uint32_t status;
	uint32_t server_instance;
} __attribute__((packed));

#define HAL_EV_MULTI_ADV_DISABLE		0xA7
struct hal_ev_multi_adv_disable {
	uint32_t status;
	uint32_t server_instance;
} __attribute__((packed));

#define HAL_EV_MULTI_ADV_UPDATE			0xA8
struct hal_ev_multi_adv_update {
	uint32_t status;
	uint32_t server_instance;
} __attribute__((packed));

#define HAL_EV_LEGACY_ADV_ENABLE		0xA9
struct hal_ev_legacy_adv_status {
	uint32_t status;
	uint32_t server_instance;
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_REGISTERED	0xAA
struct hal_ev_gatt_client_registered {
	uint32_t status;
	uint32_t client_if;
	uint8_t app_uuid[16];
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_SCAN_RESULT  0xAC
struct hal_ev_gatt_client_scan_result {
	uint8_t  bd_addr[6];
	uint8_t  addr_type;
	uint8_t  adv_type;
	int32_t  rssi;
	uint16_t len;
	uint8_t  adv_data[0];
} __attribute__((packed));

#define HAL_EV_GATT_SERVICE_ADDED		0xAD
struct hal_ev_gatt_service_added {
	uint32_t status;
	uint32_t server_instance;
	uint8_t  is_primary;
	uint8_t svc_uuid[16];
	uint32_t service_handle;
} __attribute__((packed));

#define HAL_EV_GATT_CHAR_ADDED			0xAE
struct hal_ev_gatt_char_added {
	uint32_t status;
	uint32_t server_instance;
	uint32_t service_handle;
	uint32_t char_handle;
	uint8_t char_uuid[16];
} __attribute__((packed));

#define HAL_EV_GATT_DESC_ADDED			0xAF
struct hal_ev_gatt_desc_added {
	uint32_t status;
	uint32_t server_instance;
	uint32_t service_handle;
	uint32_t desc_handle;
	uint8_t desc_uuid[16];
} __attribute__((packed));

#define HAL_EV_GATT_SERVICE_STARTED		0xB0
struct hal_ev_gatt_service_started {
	uint32_t status;
	uint32_t server_instance;
	uint32_t service_handle;
} __attribute__((packed));

#define HAL_EV_GATT_SERVICE_DELETED		0xB1
struct hal_ev_gatt_service_deleted {
	uint32_t status;
	uint32_t server_instance;
	uint32_t service_handle;
} __attribute__((packed));

#define HAL_EV_GATT_SERVER_CONNECTED		0xB2
struct hal_ev_gatt_server_connected {
	uint32_t conn_id;
	uint32_t server_instance;
	uint8_t connected;
	uint8_t bdaddr[6];
} __attribute__((packed));

#define HAL_EV_GATT_READ_REQUESTED		0xB3
struct hal_ev_gatt_server_read_req {
	uint32_t conn_id;
	uint32_t trans_id;
	uint8_t bdaddr[6];
	uint32_t att_handle;
	uint32_t offset;
	uint8_t is_long;
} __attribute__((packed));

#define HAL_EV_GATT_WRITE_REQUESTED		0xB4
struct hal_ev_gatt_server_write_req {
	uint32_t conn_id;
	uint32_t trans_id;
	uint8_t bdaddr[6];
	uint32_t att_handle;
	uint32_t offset;
	uint32_t length;
	uint8_t need_rsp;
	uint8_t is_prep;
	uint8_t value[600];
} __attribute__((packed));

#define HAL_EV_GATT_INDICATE_CFM		0xB5
struct hal_ev_gatt_server_indicate_cfm {
	uint32_t conn_id;
	uint32_t trans_id;
	uint32_t att_handle;
	uint8_t bdaddr[6];
} __attribute__((packed));

#ifdef TIZEN_BT_HAL
#define HAL_EV_GATT_NOTIFICATION_CHANGE		0xB6
struct hal_ev_gatt_server_notifcation_change {
	uint32_t conn_id;
	uint32_t trans_id;
	uint32_t att_handle;
	uint8_t notify;
	uint8_t bdaddr[6];
} __attribute__((packed));

#define HAL_EV_LE_STATE_CHANGED    0xB7
struct hal_ev_le_state_changed {
	uint8_t state;
} __attribute__((packed));
#endif /* TIZEN_BT_HAL */

#define HAL_LE_STATE_CONNECTED         0x00
#define HAL_LE_STATE_DISCONNECTED      0x01

#define HAL_EV_LE_CONN_STATE_CHANGED        0xB8
struct hal_ev_le_conn_state_changed {
	uint8_t status;
	uint8_t bdaddr[6];
	uint8_t state;
} __attribute__((packed));

#define HAL_EV_DEVICE_TRUSTED_PROFILES_CHANGED	0xB9
struct hal_ev_device_trusted_profiles_changed {
	uint8_t bdaddr[6];
	uint32_t trust_val;
} __attribute__((packed));

#define HAL_RSSI_MONITORING_ENABLED	0x00
#define HAL_RSSI_MONITORING_DISABLED	0x01

#define HAL_EV_RSSI_MONITOR_STATE_CHANGED	0xBA
struct hal_ev_rssi_monitor_state_changed {
	uint8_t bdaddr[6];
	int32_t link_type;
	uint8_t state;
} __attribute__((packed));

#define HAL_EV_RSSI_ALERT_RECIEVED	0xBB
struct hal_ev_rssi_alert_recieved {
	uint8_t bdaddr[6];
	int32_t link_type;
	int32_t alert_type;
	int32_t rssi;
} __attribute__((packed));

#define HAL_EV_RAW_RSSI_RECIEVED	0xBC
struct hal_ev_raw_rssi_recieved {
	uint8_t bdaddr[6];
	int32_t link_type;
	int32_t rssi;
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_CONNECTED	0xBD
#define HAL_EV_GATT_CLIENT_DISCONNECTED  0xBE
struct hal_ev_gatt_client_connected {
	int32_t conn_id;
	int32_t status;
	int32_t client_if;
	uint8_t bdaddr[6];
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_SEARCH_RESULT	0xBF
struct hal_ev_gatt_client_search_result {
	int32_t conn_id;
	int32_t inst_id;
	uint8_t uuid[16];
	int32_t is_primary;
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_SEARCH_COMPLETE	0xC0
struct hal_ev_gatt_client_search_complete {
	int32_t conn_id;
	int32_t status;
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_CHARAC_SEARCH_RESULT	0xC1
struct hal_ev_gatt_client_char_search_result {
	int32_t conn_id;
	int32_t status;
	int32_t is_primary;
	int32_t inst_id;
	uint8_t svc_uuid[16];
	uint8_t char_uuid[16];
	int32_t char_prop;
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_DESC_SEARCH_RESULT 0XC2
struct hal_ev_gatt_client_desc_search_result {
	int32_t conn_id;
	int32_t status;
	int32_t is_primary;
	int32_t inst_id;
	uint8_t svc_uuid[16];
	uint8_t char_uuid[16];
	uint8_t desc_uuid[16];
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_READ_CHARAC	0XC3
#define HAL_EV_GATT_CLIENT_READ_DESC	0XC4
struct hal_ev_gatt_client_read_data {
	int32_t conn_id;
	int32_t status;
	int32_t is_primary;
	int32_t inst_id;
	uint8_t svc_uuid[16];
	uint8_t char_uuid[16];
	uint8_t desc_uuid[16];
	uint8_t value[600];
	int32_t len;
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_WRITE_CHARAC	0XC5
#define HAL_EV_GATT_CLIENT_WRITE_DESC	0XC6
struct hal_ev_gatt_client_write_result {
	int32_t conn_id;
	int32_t status;
	int32_t is_primary;
	int32_t inst_id;
	uint8_t svc_uuid[16];
	uint8_t char_uuid[16];
	uint8_t desc_uuid[16];
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_WATCH_NOTIFICATION  0XC7
struct hal_ev_gatt_client_watch_notification {
	int32_t client_if;
	int32_t registered;
	int32_t status;
	int32_t is_primary;
	int32_t inst_id;
	uint8_t svc_uuid[16];
	uint8_t char_uuid[16];
} __attribute__((packed));

#define HAL_EV_GATT_ACQUIRE_WRITE  0XC8
struct hal_ev_gatt_acquire_write {
	int32_t fd;
	int32_t conn_id;
	int32_t status;
	uint8_t svc_uuid[16];
	uint8_t char_uuid[16];
} __attribute__((packed));

#define HAL_EV_GATT_SERVER_ACQUIRE_WRITE_RES  0XC9
struct hal_ev_gatt_server_acquire_write_res {
	int32_t mtu;
	int32_t trans_id;
	int32_t conn_id;
	int32_t char_handl;
	uint8_t bdaddr[6];
} __attribute__((packed));

#define HAL_EV_GATT_SERVER_ACQUIRE_NOTIFY_RES  0XCA
struct hal_ev_gatt_server_acquire_notify {
	int32_t mtu;
	int32_t trans_id;
	int32_t conn_id;
	int32_t char_handl;
	char *path;
} __attribute__((packed));

#define HAL_EV_GATT_CLIENT_NOTIFY_CHANGED_VALUE 0XCB
struct hal_ev_gatt_client_notify_changed_value {
	int32_t conn_id;
	uint8_t value[600];
	int32_t len;
	uint8_t bdaddr[6];
	int32_t is_primary;
	int32_t inst_id;
	uint8_t svc_uuid[16];
	uint8_t char_uuid[16];
	uint8_t is_notify;
} __attribute__((packed));

#endif //_BT_HAL_MSG_H_
