/*
 * Copyright (C) 2013 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _BT_HAL_UTILS_H_
#define _BT_HAL_UTILS_H_

#include <endian.h>

#include <hardware/bluetooth.h>

#define MAX_UUID_STR_LEN	37
#define HAL_UUID_LEN		16
#define MAX_ADDR_STR_LEN	18
#define MAX_MANUFACTURE_LEN	240

#define BT_HAL_UUID_16     2
#define BT_HAL_UUID_32     4
#define BT_HAL_UUID_128    16

#define CASE_RETURN_STR(const) case const: return #const;

static const char BT_BASE_UUID[BT_HAL_UUID_128] = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00,
	0x80, 0x00, 0x00, 0x80, 0x5f, 0x9b, 0x34, 0xfb
};

static const char BASE_UUID_CONVERTED[BT_HAL_UUID_128] = {
	0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80,
	0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

/** BT Profile Service UUID's */
#define BT_HAL_HFP_AUDIO_GATEWAY_UUID "0000111f-0000-1000-8000-00805f9b34fb"
#define BT_HAL_HSP_AUDIO_GATEWAY_UUID "00001112-0000-1000-8000-00805f9b34fb"
#define BT_HAL_HFP_HF_UUID "0000111e-0000-1000-8000-00805f9b34fb"
#define BT_HAL_HSP_HS_UUID "00001108-0000-1000-8000-00805f9b34fb"
#define BT_HAL_A2DP_PROFILE_UUID "0000110d-0000-1000-8000-00805f9b34fb" /* This is A2DP Profile UUID */
#define BT_HAL_A2DP_SOURCE_UUID "0000110a-0000-1000-8000-00805f9b34fb"  /* This is A2DP Profile Role(SRC) UUID */
#define BT_HAL_A2DP_SINK_UUID "0000110b-0000-1000-8000-00805f9b34fb"   /* This is A2DP Profile Role(SNK) UUID */
#define BT_HAL_AVRCP_TARGET_UUID "0000110c-0000-1000-8000-00805f9b34fb"
#define BT_HAL_AVRCP_REMOTE_UUID "0000110e-0000-1000-8000-00805f9b34fb"
#define BT_HAL_OPP_UUID "00001105-0000-1000-8000-00805f9b34fb"
#define BT_HAL_FTP_UUID "00001106-0000-1000-8000-00805f9b34fb"
#define BT_HAL_SPP_UUID "00001101-0000-1000-8000-00805f9b34fb"
#define BT_HAL_PBAP_UUID "0000112f-0000-1000-8000-00805f9b34fb"
#define BT_HAL_MAP_UUID "00001132-0000-1000-8000-00805f9b34fb"
#define BT_HAL_NAP_UUID "00001116-0000-1000-8000-00805f9b34fb"
#define BT_HAL_GN_UUID "00001117-0000-1000-8000-00805f9b34fb"
#define BT_HAL_BNEP_UUID "0000000f-0000-1000-8000-00805f9b34fb"
#define BT_HAL_HID_UUID "00001124-0000-1000-8000-00805f9b34fb"
#define BT_HAL_SAP_UUID_OLD "a49eb41e-cb06-495c-9f4f-bb80a90cdf00"
#define BT_HAL_SAP_UUID_NEW "a49eb41e-cb06-495c-9f4f-aa80a90cdf4a"

#ifdef TIZEN_BT_HAL
#define BT_HAL_HID_DEVICE_UUID "00001124-0000-1000-8000-00805f9b43bf"
#define BT_HAL_IOTIVITY_UUID "12341234-1c25-481f-9dfb-59193d238280"
#endif

// List of Protocols
#define BASE                                    0x0000          //Base
#define SDP                                     0x0001          //Bluetooth Core Specification
#define UDP                                     0x0002          //[NO USE BY PROFILES]
#define RFCOMM                                  0x0003          //RFCOMM with TS 07.10
#define TCP                                     0x0004          //[NO USE BY PROFILES]
#define TCS_BIN                                 0x0005          //Telephony Control Specification / TCS Binary [DEPRECATED]
#define TCS_AT                                  0x0006          //[NO USE BY PROFILES]
#define ATT                                     0x0007          //Attribute Protocol
#define OBEX                                    0x0008          //IrDA Interoperability
#define IP                                      0x0009          //[NO USE BY PROFILES]
#define FTP                                     0x000A          //[NO USE BY PROFILES]
#define HTTP                                    0x000C          //[NO USE BY PROFILES]
#define WSP                                     0x000E          //[NO USE BY PROFILES]
#define BNEP                                    0x000F          //Bluetooth Network Encapsulation Protocol (BNEP)
#define UPNP                                    0x0010          //Extended Service Discovery Profile (ESDP) [DEPRECATED]
#define HIDP                                    0x0011          //Human Interface Device Profile (HID)
#define HardcopyControlChannel                  0x0012          //Hardcopy Cable Replacement Profile (HCRP)
#define HardcopyDataChannel                     0x0014          //See Hardcopy Cable Replacement Profile (HCRP)
#define HardcopyNotification                    0x0016          //Hardcopy Cable Replacement Profile (HCRP)
#define AVCTP                                   0x0017          //Audio/Video Control Transport Protocol (AVCTP)
#define AVDTP                                   0x0019          //Audio/Video Distribution Transport Protocol (AVDTP)
#define CMTP                                    0x001B          //Common ISDN Access Profile (CIP) [DEPRECATED]
#define MCAPControlChannel                      0x001E          //Multi-Channel Adaptation Protocol (MCAP)
#define MCAPDataChannel                         0x001F          //Multi-Channel Adaptation Protocol (MCAP)
#define L2CAP                                   0x0100          //Bluetooth Core Specification

//List of Services
#define ServiceDiscoveryServerServiceClassID    0x1000          //Bluetooth Core Specification Service Class
#define BrowseGroupDescriptorServiceClassID     0x1001          //Bluetooth Core Specification Service Class
#define SerialPort                              0x1101          //Serial Port Profile (SPP)
#define LANAccessUsingPPP                       0x1102          //LAN Access Profile
#define DialupNetworking                        0x1103          //Dial-up Networking Profile (DUN)
#define IrMCSync                                0x1104          //Synchronization Profile (SYNC)
#define OBEXObjectPush                          0x1105          //Object Push Profile (OPP)
#define OBEXFileTransfer                        0x1106          //File Transfer Profile (FTP)
#define IrMCSyncCommand                         0x1107          //Synchronization Profile (SYNC)
#define Headset                                 0x1108          //Headset Profile (HSP)
#define CordlessTelephony                       0x1109          //Cordless Telephony Profile (CTP)
#define AudioSource                             0x110A          //Advanced Audio Distribution Profile (A2DP) Service Class
#define AudioSink                               0x110B          //Advanced Audio Distribution Profile (A2DP) Service Class
#define AV_RemoteControlTarget                  0x110C          //Audio/Video Remote Control Profile (AVRCP) Service Class
#define AdvancedAudioDistribution               0x110D          //Advanced Audio Distribution Profile (A2DP) Profile
#define AV_RemoteControl                        0x110E          //Audio/Video Remote Control Profile (AVRCP)
#define AV_RemoteControlController              0x110F          //Audio/Video Remote Control Profile (AVRCP)
#define Intercom                                0x1110          //Intercom Profile (ICP)
#define Fax                                     0x1111          //Fax Profile (FAX)
#define Headset_Audio_Gateway                   0x1112          //Headset Profile (HSP) Service Class
#define WAP                                     0x1113          //Interoperability Requirements for Bluetooth technology as a WAP, Bluetooth SIG [DEPRECATED] Service Class
#define WAP_CLIENT                              0x1114          //Interoperability Requirements for Bluetooth technology as a WAP, Bluetooth SIG [DEPRECATED] Service Class
#define PANU                                    0x1115          //Personal Area Networking Profile (PAN)
#define NAP                                     0x1116          //Personal Area Networking Profile (PAN)
#define GN                                      0x1117          //Personal Area Networking Profile (PAN)
#define DirectPrinting                          0x1118          //Basic Printing Profile (BPP) Service Class
#define ReferencePrinting                       0x1119          //See Basic Printing Profile (BPP) Service Class
#define Basic_Imaging_Profile                   0x111A          //Basic Imaging Profile (BIP) Profile
#define ImagingResponder                        0x111B          //Basic Imaging Profile (BIP) Service Class
#define ImagingAutomaticArchive                 0x111C          //Basic Imaging Profile (BIP) Service Class
#define ImagingReferencedObjects                0x111D          //Basic Imaging Profile (BIP) Service Class
#define Handsfree                               0x111E          //Hands-Free Profile (HFP)
#define HandsfreeAudioGateway                   0x111F          //Hands-free Profile (HFP) Service Class
#define DirectPrintingReferenceObjectsService   0x1120          //Basic Printing Profile (BPP) Service Class
#define ReflectedUI                             0x1121          //Basic Printing Profile (BPP) Service Class
#define BasicPrinting                           0x1122          //Basic Printing Profile (BPP) Profile
#define PrintingStatus                          0x1123          //Basic Printing Profile (BPP) Service Class
#define HumanInterfaceDeviceService             0x1124          //Human Interface Device (HID)
#define HardcopyCableReplacement                0x1125          //Hardcopy Cable Replacement Profile (HCRP) Profile
#define HCR_Print                               0x1126          //Hardcopy Cable Replacement Profile (HCRP) Service Class
#define HCR_Scan                                0x1127          //Hardcopy Cable Replacement Profile (HCRP) Service Class
#define Common_ISDN_Access                      0x1128          //Common ISDN Access Profile (CIP)
#define SIM_Access                              0x112D          //SIM Access Profile (SAP)
#define Phonebook_Access_PCE                    0x112E          //Phonebook Access Profile (PBAP) Service Class
#define Phonebook_Access_PSE                    0x112F          //Phonebook Access Profile (PBAP) Service Class
#define Phonebook_Access                        0x1130          //Phonebook Access Profile (PBAP) Profile
#define Headset_HS                              0x1131          //Headset Profile (HSP)
#define Message_Access_Server                   0x1132          //Message Access Profile (MAP) Service Class
#define Message_Notification_Server             0x1133          //Message Access Profile (MAP) Service Class
#define Message_Access_Profile                  0x1134          //Message Access Profile (MAP) Profile
#define GNSS                                    0x1135          //Global Navigation Satellite System Profile (GNSS) Profile
#define GNSS_Server                             0x1136          // Global Navigation Satellite System Profile (GNSS) Service Class
#define ThreeD_Display                          0x1137          //3D Synchronization Profile (3DSP) Service Class?
#define ThreeD_Glasses                          0x1138          //3D Synchronization Profile (3DSP) ?Service Class
#define ThreeD_Synchronization                  0x1139          //3D Synchronization Profile (3DSP) ?Profile
#define MPS_Profile                             0x113A          //Multi-Profile Specification (MPS) ?Profile
#define MPS_SC                                  0x113B          //Multi-Profile Specification (MPS) ?Service Class
#define CTN_Access_Service                      0x113C          //Calendar, Task, and Notes (CTN) Profile ?Service Class
#define CTN_Notification_Service                0x113D          //Calendar Tasks and Notes (CTN) Profile ?Service Class
#define CTN_Profile                             0x113E          //Calendar Tasks and Notes (CTN) Profile ?Profile
#define PnPInformation                          0x1200          //Device Identification (DID)
#define GenericNetworking                       0x1201          //N/A Service Class
#define GenericFileTransfer                     0x1202          //N/A Service Class
#define GenericAudio                            0x1203          //N/A Service Class
#define GenericTelephony                        0x1204          //N/A Service Class
#define UPNP_Service                            0x1205          //Enhanced Service Discovery Profile (ESDP) [DEPRECATED] Service Class
#define UPNP_IP_Service                         0x1206          //Enhanced Service Discovery Profile (ESDP) [DEPRECATED] Service Class
#define ESDP_UPNP_IP_PAN                        0x1300          //Enhanced Service Discovery Profile (ESDP) [DEPRECATED] Service Class
#define ESDP_UPNP_IP_LAP                        0x1301          //Enhanced Service Discovery Profile (ESDP)[DEPRECATED] Service Class
#define ESDP_UPNP_L2CAP                         0x1302          //Enhanced Service Discovery Profile (ESDP)[DEPRECATED] Service Class
#define VideoSource                             0x1303          //Video Distribution Profile (VDP) Service Class
#define VideoSink                               0x1304          //Video Distribution Profile (VDP) Service Class
#define VideoDistribution                       0x1305          //Video Distribution Profile (VDP) Profile
#define HDP                                     0x1400          //Health Device Profile Profile
#define HDP_Source                              0x1401          //Health Device Profile (HDP) Service Class
#define HDP_Sink                                0x1402          //Health Device Profile (HDP) Service Class

/* List of GATT Services UUID */
#define Generic_Access							0x1800			//Generic Access Service
#define Generic_Attribute						0x1801			//Generic Attribute Service
#define Immediate_Alert							0x1802			//Immediate Alert Service
#define Link_Loss								0x1803			//Link Loss Service
#define Tx_Power								0x1804			//Tx Power Service
#define Current_Time							0x1805			//Current Time Service
#define Reference_Time_Update					0x1806			//Reference Time Update Service
#define Next_Dst_Change							0x1807			//Next Destination Change Service
#define Glucose									0x1808			//Glucose Service
#define Health_Thermometer						0x1809			//Health Thermometer Service
#define Device_Information						0x180A			//Device Information Service
#define Heart_Rate								0x180D			//Heart Rate Service
#define Phone_Alert_Status						0x180E			//Phone Alert Status Service
#define Battery_Service							0x180F			//Battery Service
#define Blood_Pressure							0x1810			//Blood Pressure Service
#define Alert_Notification						0x1811			//Alert Notification Service
#define Human_Interface_Device					0x1812			//HID Service
#define Scan_Parameters							0x1813			//Scan Parameters Service
#define Running_Speed_And_Cadence				0x1814			//Running Speed and Cadence Service
#define Automation_IO							0x1815			//Automation IO Service
#define Cycling_Speed_And_Cadence				0x1816			//Cycling Speed and Cadence Service
#define Cycling_Power							0x1818			//Cycling Power Service
#define Location_And_Navigation					0x1819			//Location and Navigation Service
#define Environmental_Sensing					0x181A			//Environment Sensing Service
#define Body_Composition						0x181B			//Body Composition Service
#define User_Data								0x181C			//User Data Service
#define Weight_Scale							0x181D			//Weight Scale Service
#define Bond_Management							0x181E			//Bond Management Service
#define Continuous_Glucose_Monitoring			0x181F			//Continuous Glucose Monitoring Service
#define Internet_Protocol_Support				0x1820			//Internet Protocol Support Service
#define Indoor_Positioning						0x1821			//Indoor Positioning Service
#define Pulse_Oximeter							0x1822			//Pulse Oximeter Service
#define Http_Proxy								0x1823			//Http Proxy Service
#define Transport_Discovery						0x1824			//Transport Discovery Service
#define Object_Transfer							0x1825			//Object Transfer Service

/* List of GATT Characteristics UUID */
#define Gap_Device_Name										0x2A00
#define Gap_Appearance										0x2A01
#define Gap_Peripheral_Privacy_Flag							0x2A02
#define Gap_Reconnection_Address							0x2A03
#define Gap_Peripheral_Preferred_Connection_Parameters		0x2A04
#define Gatt_Service_Changed								0x2A05
#define Alert_Level											0x2A06
#define Tx_Power_Level										0x2A07
#define Date_Time											0x2A08
#define Day_Of_Week											0x2A09
#define Day_Date_Time										0x2A0A
#define Exact_Time_256										0x2A0C
#define Dst_Offset											0x2A0D
#define Time_Zone											0x2A0E
#define Local_Time_Information								0x2A0F
#define Time_With_Dst										0x2A11
#define Time_Accuracy										0x2A12
#define Time_Source											0x2A13
#define Reference_Time_Information							0x2A14
#define Time_Update_Control_Point							0x2A16
#define Time_Update_State									0x2A17
#define Glucose_Measurement									0x2A18
#define Battery_Level										0x2A19
#define Temperature_Measurement								0x2A1C
#define Temperature_Type									0x2A1D
#define Intermediate_Temperature							0x2A1E
#define Measurement_Interval								0x2A21
#define Boot_Keyboard_Input_Report							0x2A22
#define System_Id											0x2A23
#define Model_Number_String									0x2A24
#define Serial_Number_String								0x2A25
#define Firmware_Revision_String							0x2A26
#define Hardware_Revision_String							0x2A27
#define Software_Revision_String							0x2A28
#define Manufacturer_Name_String							0x2A29
#define Regulatory_Certification_Data_List					0x2A2A
#define Current_Time_Charac									0x2A2B
#define Magnetic_Declination								0x2A2C
#define Scan_Refresh										0x2A31
#define Boot_Keyboard_Output_Report							0x2A32
#define Boot_Mouse_Input_Report								0x2A33
#define Glucose_Measurement_Context							0x2A34
#define Blood_Pressure_Measurement							0x2A35
#define Intermediate_Cuff_Pressure							0x2A36
#define Heart_Rate_Measurement								0x2A37
#define Body_Sensor_Location								0x2A38
#define Heart_Rate_Control_Point							0x2A39
#define Alert_Status										0x2A3F
#define Ringer_Control_Point								0x2A40
#define Ringer_Setting										0x2A41
#define Alert_Category_Id_Bit_Mask							0x2A42
#define Alert_Category_Id									0x2A43
#define Alert_Notification_Control_Point					0x2A44
#define Unread_Alert_Status									0x2A45
#define New_Alert											0x2A46
#define Supported_New_Alert_Category						0x2A47
#define Supported_Unread_Alert_Category						0x2A48
#define Blood_Pressure_Feature								0x2A49
#define Hid_Information										0x2A4A
#define Report_Map											0x2A4B
#define Hid_Control_Point									0x2A4C
#define Report												0x2A4D
#define Protocol_Mode										0x2A4E
#define Scan_Interval_Window								0x2A4F
#define Pnp_Id												0x2A50
#define Glucose_Feature										0x2A51
#define Record_Access_Control_Point							0x2A52
#define Rsc_Measurement										0x2A53
#define Rsc_Feature											0x2A54
#define Sc_Control_Point									0x2A55
#define Digital												0x2A56
#define Analog												0x2A58
#define Aggregate											0x2A5A
#define Csc_Measurement										0x2A5B
#define Csc_Feature											0x2A5C
#define Sensor_Location										0x2A5D
#define Plx_Spot_Check_Measurement							0x2A5E
#define Plx_Continuous_Measurement							0x2A5F
#define Plx_Features										0x2A60
#define Cycling_Power_Measurement							0x2A63
#define Cycling_Power_Vector								0x2A64
#define Cycling_Power_Feature								0x2A65
#define Cycling_Power_Control_Point							0x2A66
#define Location_And_Speed									0x2A67
#define Navigation											0x2A68
#define Position_Quality									0x2A69
#define Ln_Feature											0x2A6A
#define Ln_Control_Point									0x2A6B
#define Elevation											0x2A6C
#define Pressure											0x2A6D
#define Temperature											0x2A6E
#define Humidity											0x2A6F
#define True_Wind_Speed										0x2A70
#define True_Wind_Direction									0x2A71
#define Apparent_Wind_Speed									0x2A72
#define Apparent_Wind_Direction								0x2A73
#define Gust_Factor											0x2A74
#define Pollen_Concentration								0x2A75
#define Uv_Index											0x2A76
#define Irradiance											0x2A77
#define Rainfall											0x2A78
#define Wind_Chill											0x2A79
#define Heat_Index											0x2A7A
#define Dew_Point											0x2A7B
#define Descriptor_Value_Changed							0x2A7D
#define Aerobic_Threshold									0x2A7F
#define Age													0x2A80
#define Anaerobic_Heart_Rate_Lower_Limit					0x2A81
#define Anaerobic_Heart_Rate_Upper_Limit					0x2A82
#define Anaerobic_Threshold									0x2A83
#define Aerobic_Heart_Rate_Upper_Limit						0x2A84
#define Date_Of_Birth										0x2A85
#define Date_Of_Threshold_Assessment						0x2A86
#define Email_Address										0x2A87
#define Fat_Burn_Heart_Rate_Lower_Limit						0x2A88
#define Fat_Burn_Heart_Rate_Upper_Limit						0x2A89
#define First_Name											0x2A8A
#define Five_Zone_Heart_Rate_Limits							0x2A8B
#define Gender												0x2A8C
#define Heart_Rate_Max										0x2A8D
#define Height												0x2A8E
#define Hip_Circumference									0x2A8F
#define Last_Name											0x2A90
#define Maximum_Recommended_Heart_Rate						0x2A91
#define Resting_Heart_Rate									0x2A92
#define Sport_Type_For_Aerobic_And_Anaerobic_Thresholds		0x2A93
#define Three_Zone_Heart_Rate_Limits						0x2A94
#define Two_Zone_Heart_Rate_Limit							0x2A95
#define Vo2_Max												0x2A96
#define Waist_Circumference									0x2A97
#define Weight												0x2A98
#define Database_Change_Increment							0x2A99
#define User_Index											0x2A9A
#define Body_Composition_Feature							0x2A9B
#define Body_Composition_Measurement						0x2A9C
#define Weight_Measurement									0x2A9D
#define Weight_Scale_Feature								0x2A9E
#define User_Control_Point									0x2A9F
#define Magnetic_Flux_Density_2D							0x2AA0
#define Magnetic_Flux_Density_3D							0x2AA1
#define Language											0x2AA2
#define Barometric_Pressure_Trend							0x2AA3
#define Bond_Management_Control_Point						0x2AA4
#define Bond_Management_Feature								0x2AA5
#define Gap_Central_Address_Resolution_Support				0x2AA6
#define Cgm_Measurement										0x2AA7
#define Cgm_Feature											0x2AA8
#define Cgm_Status											0x2AA9
#define Cgm_Session_Start_Time								0x2AAA
#define Cgm_Session_Run_Time								0x2AAB
#define Cgm_Specific_Ops_Control_Point						0x2AAC
#define Indoor_Positioning_Configuration					0x2AAD
#define Latitude											0x2AAE
#define Longitude											0x2AAF
#define Local_North_Coordinate								0x2AB0
#define Local_East_Coordinate								0x2AB1
#define Floor_Number										0x2AB2
#define Altitude											0x2AB3
#define Uncertainty											0x2AB4
#define Location_Name										0x2AB5
#define Uri													0x2AB6
#define Http_Headers										0x2AB7
#define Http_Status_Code									0x2AB8
#define Http_Entity_Body									0x2AB9
#define Http_Control_Point									0x2ABA
#define Https_Security										0x2ABB
#define Tds_Control_Point									0x2ABC
#define Ots_Feature											0x2ABD
#define Object_Name											0x2ABE
#define Object_Type											0x2ABF
#define Object_Size											0x2AC0
#define Object_First_Created								0x2AC1
#define Object_Last_Modified								0x2AC2
#define Object_Id											0x2AC3
#define Object_Properties									0x2AC4
#define Object_Action_Control_Point							0x2AC5
#define Object_List_Control_Point							0x2AC6
#define Object_List_Filter									0x2AC7
#define Object_Changed										0x2AC8
#define Fitness_Machine_Control_Point						0xEE1D

/* List of GATT Descriptors UUID */
#define Gatt_Characteristic_Extended_Properties			0x2900
#define Gatt_Characteristic_User_Description			0x2901
#define Gatt_Client_Characteristic_Configuration		0x2902
#define Gatt_Server_Characteristic_Configuration		0x2903
#define Gatt_Characteristic_Presentation_Format			0x2904
#define Gatt_Characteristic_Aggregate_Format			0x2905
#define Valid_Range										0x2906
#define External_Report_Reference						0x2907
#define Report_Reference								0x2908
#define Number_Of_Digitals								0x2909
#define Value_Trigger_Setting							0x290A
#define Es_Configuration								0x290B
#define Es_Measurement									0x290C
#define Es_Trigger_Setting								0x290D
#define Time_Trigger_Setting							0x290E

/* List of GATT Declarations UUID */
#define Gatt_Primary_Service_Declaration				0x2800
#define Gatt_Secondary_Service_Declaration				0x2801
#define Gatt_Include_Declaration						0x2802
#define Gatt_Characteristic_Declaration					0x2803

const char *bt_uuid_t2str(const uint8_t *uuid, char *buf);
const char *btuuid2str(const uint8_t *uuid);
const char *bt_bdaddr_t2str(const bt_bdaddr_t *bd_addr, char *buf);
void str2bt_bdaddr_t(const char *str, bt_bdaddr_t *bd_addr);
void str2bt_uuid_t(const char *str, bt_uuid_t *uuid);
const char *btproperty2str(const bt_property_t *property);
const char *bdaddr2str(const bt_bdaddr_t *bd_addr);

const char* bt_property_type_t2str(bt_property_type_t prop_type);
const char* bt_device_type_t2str(bt_device_type_t device_type);

struct int2str {
	int val;		/* int value */
	const char *str;	/* corresponding string */
};

int int2str_findint(int v, const struct int2str m[]);
int int2str_findstr(const char *str, const struct int2str m[]);
const char *enum_defines(void *v, int i);
const char *enum_strings(void *v, int i);
const char *enum_one_string(void *v, int i);
bt_service_id_t _bt_convert_uuid_string_to_service_id(const char *uuid);
char *_bt_convert_service_id_to_uuid_string(bt_service_id_t service_id);
const char *_bt_hal_dump_uuid_name(int uuid_no);

#endif //_BT_HAL_UTILS_H_
