/*
 * BLUETOOOTH HAL
 *
 * Copyright (c) 2015 - 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact:  Anupam Roy <anupam.r@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _BT_HAL_ADAPTER_LE_H_
#define _BT_HAL_ADAPTER_LE_H_

#include <glib.h>
#include <sys/types.h>

#include <hardware/bt_gatt_server.h>

#include "bt-hal-event-receiver.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef TIZEN_BT_HAL
int _bt_hal_le_enable(void);

int _bt_hal_le_disable(void);
#endif

void _bt_hal_register_gatt_le_dbus_handler_cb(handle_stack_msg cb);

void _bt_hal_unregister_gatt_le_dbus_handler_cb();

gboolean _bt_hal_update_le_feature_support(const char *item, const char *value,
                        bt_local_le_features_t *le_features);

int _bt_hal_get_available_adv_slot_id(bt_uuid_t *uuid);

int _bt_hal_get_adv_slot_adv_handle(int slot_id);

void _bt_hal_unregister_adv_slot_owner(int slot_id);

gboolean _bt_hal_is_advertising_in_slot(int slot);

void _bt_hal_set_advertising_status(int slot_id, gboolean mode);

gboolean _bt_hal_is_advertising(void);

void _bt_hal_free_server_slot(int slot_id);

int _bt_hal_set_advertising_data(btgatt_adv_param_setup_t adv_param_setup);

int _bt_hal_set_advertising_params(int server_if, int min_interval,
		int max_interval, int adv_type,
		int chnl_map, int tx_power, int timeout_s);

int _bt_hal_enable_advertising(int server_if, bool enable, bool is_multi_adv);

int _bt_hal_adapter_le_start_scan(void);

int _bt_hal_adapter_le_stop_scan(void);

int _bt_hal_adapter_le_set_scan_parameters(
		int scan_type, int scan_interval, int scan_window);

void _bt_hal_get_gatt_server_instance_initialized(int *instance);

int _bt_hal_le_init(void);
void _bt_hal_le_deinit(void);
#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_BT_HAL_ADAPTER_LE_H_*/
