/* Bluetooth-frwk
 *
 * Copyright (c) 2015 - 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Nilesh Trimbake <t.shripati@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef _BT_HAL_AVRCP_CTRL_DBUS_HANDLER_H_
#define _BT_HAL_AVRCP_CTRL_DBUS_HANDLER_H_

#include <glib.h>
#include <sys/types.h>

#include "bt-hal.h"
#include "bt-hal-log.h"
#include "bt-hal-msg.h"

#include "bt-hal-event-receiver.h"

#ifdef __cplusplus
extern "C" {
#endif
typedef enum {
	PLAY = 0x01,
	PAUSE,
	STOP,
	NEXT,
	PREVIOUS,
	PRESS_FAST_FORWARD,
	RELEASE_FAST_FORWARD,
	PRESS_REWIND,
	RELEASE_REWIND,
	VOLUME_UP,
	VOLUME_DOWN
} media_player_control_cmd;

typedef enum {
	EQUALIZER_OFF = 0x01,
	EQUALIZER_ON,
	EQUALIZER_INVALID,
} media_player_equalizer_status;

typedef enum {
	REPEAT_MODE_OFF = 0x01,
	REPEAT_SINGLE_TRACK,
	REPEAT_ALL_TRACK,
	REPEAT_GROUP,
	REPEAT_INVALID,
} media_player_repeat_status;

typedef enum {
	SHUFFLE_MODE_OFF = 0x01,
	SHUFFLE_ALL_TRACK,
	SHUFFLE_GROUP,
	SHUFFLE_INVALID,
} media_player_shuffle_status;

typedef enum {
	SCAN_MODE_OFF = 0x01,
	SCAN_ALL_TRACK,
	SCAN_GROUP,
	SCAN_INVALID,
} media_player_scan_status;

typedef enum {
	STATUS_STOPPED = 0x00,
	STATUS_PLAYING,
	STATUS_PAUSED,
	STATUS_FORWARD_SEEK,
	STATUS_REVERSE_SEEK,
	STATUS_ERROR,
	STATUS_INVALID
} media_player_status;

typedef struct {
	int key;
	const char *property;
} bt_player_settinngs_t;

void _bt_hal_register_avrcp_ctrl_dbus_handler_cb(handle_stack_msg cb);

void _bt_hal_unregister_avrcp_ctrl_dbus_handler_cb();

bt_status_t _bt_hal_dbus_handler_avrcp_ctrl_connect(bt_bdaddr_t *bd_addr);

bt_status_t _bt_hal_dbus_handler_avrcp_ctrl_disconnect(bt_bdaddr_t *bd_addr);

bt_status_t _bt_hal_dbus_handler_avrcp_ctrl_cmd(bt_bdaddr_t *bd_addr, uint8_t key_code, uint8_t key_state);

bt_status_t _bt_hal_dbus_handler_avrcp_ctrl_set_property(bt_bdaddr_t *bd_addr, uint8_t type, uint8_t value);

bt_status_t _bt_hal_dbus_handler_avrcp_transport_set_property(bt_bdaddr_t *bd_addr, int type, unsigned int value);

int _bt_media_attr_to_type(const char *str);

int _bt_media_attrval_to_val(int type, const char *value);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* _BT_HAL_AVRCP_CTRL_DBUS_HANDLER_H_*/
