/*
 * Open Adaptation Layer (OAL)
 *
 * Copyright (c) 2014-2015 Samsung Electronics Co., Ltd.
 *
 * Contact: Nilesh Trimbake <t.shripati@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef _OAL_A2DP_SINK_H_
#define _OAL_A2DP_SINK_H_


#include <glib.h>
#include <sys/types.h>
#include <oal-manager.h>
#include "oal-event.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Enable Audio Host(A2DP Sink) Feature
 *
 * @remarks  BT Sound Share/BT speaker will be able to connect.
 *
 * @return OAL_STATUS_SUCCESS on success, otherwise non-zero error value.
 * @retval #OAL_STATUS_SUCCESS	Successful
 *
 * @pre OAL API must be initialized with oal_bt_init().
 *
 * @see  a2dp_sink_disable()
 */
oal_status_t a2dp_sink_enable(char *service_name , char *provider_name);


/**
 * @brief Disable Audio Host(A2DP Sink) Feature
 *
 * @remarks  BT Sound Share/BT speaker will be able to connect.
 *
 * @return OAL_STATUS_SUCCESS on success, otherwise a non-zero error value.
 * @retval #OAL_STATUS_SUCCESS	Successful
 *
 * @pre Audio host should be enabled with a2dp_sink_enable().
 *
 * @see  a2dp_sink_enable()
 */
oal_status_t a2dp_sink_disable(void);


/**
 * @brief Initiate a connection with A2DP Source Device
 *
 * @details  Result will be notified through an OAL event
 *
 * @return OAL_STATUS_SUCCESS on success, otherwise a non-zero error value.
 * @retval #OAL_STATUS_SUCCESS	Successful
 *
 * @pre BT Audio should be enabled with a2dp_sink_enable().
 *
 * @see  OAL_EVENT_A2DP_SINK_CONNECTED
 */

oal_status_t a2dp_sink_connect(bt_address_t *device_address);


/**
 * @brief Remove a connection with A2DP Source device
 *
 * @details  Result will be notified through an OAL event
 *
 * @return OAL_STATUS_SUCCESS on success, otherwise a non-zero error value.
 * @retval #OAL_STATUS_SUCCESS	Successful
 *
 * @pre BT Audio should be connected with a BT Sound device.
 *
 * @see  OAL_EVENT_A2DP_SINK_DISCONNECTED
 */

oal_status_t a2dp_sink_disconnect(bt_address_t *device_address);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_OAL_A2DP_SINK_H_*/
