/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#define BT_OTP_SERVICE_NAME	"org.projectx.otp"
#define BT_OTP_OBJECT_PATH	"/org/projectx/otp"
#define BT_OTP_INTERFACE_NAME	"org.projectx.otp_service"

#define BLE_DISABLED		"LeDisabled"
#define PROPERTIES_CHANGED	"PropertiesChanged"
#define OTC_DISCONNECTED	"OtcDisconnected"

#define BT_MANAGER_PATH "/"
#define BT_MANAGER_INTERFACE "org.freedesktop.DBus.ObjectManager"
#define BT_BLUEZ_NAME "org.bluez"
#define BT_DEVICE_INTERFACE "org.bluez.Device1"
#define BT_PROPERTIES_INTERFACE "org.freedesktop.DBus.Properties"

#define BT_OTC_INTERFACE_NAME	"org.projectx.otc_channel"

#define HEADED_PLUGIN_FILEPATH "/usr/lib/bt-plugin-headed.so"
#define HEADED_PLUGIN_FILEPATH64 "/usr/lib64/bt-plugin-headed.so"
#define FILEPATH_ARCH_64 "/usr/lib64"

#define BT_FILE_PATH_MAX_LEN 262
#define BT_L2CAP_BUFFER_LEN 672

#define BT_ADDRESS_STRING_SIZE 18

#define BT_OTP_BASE_DIR_PATH "/home/owner/media/otp/"

#define BT_OACP_MAX_TIMEOUT 10000 /* Timeout for OACP operation in msec */

/* OTP Service and Chanracteristics UUID */
#define OTP_UUID			"1825"
#define OTP_FEATURE_UUID		"2abd"
#define OTP_OBJECT_NAME_UUID		"2abe"
#define OTP_OBJECT_TYPE_UUID		"2abf"
#define OTP_OBJECT_SIZE_UUID		"2ac0"
#define OTP_OBJECT_FIRST_CREATED_UUID	"2ac1"
#define OTP_OBJECT_LAST_MODIFIED_UUID	"2ac2"
#define OTP_OBJECT_ID_UUID		"2ac3"
#define OTP_OBJECT_PROP_UUID		"2ac4"
#define OTP_OACP_UUID			"2ac5"
#define OTP_OLCP_UUID			"2ac6"
#define OTP_OBJECT_LIST_FILTER_UUID	"2ac7"
#define OTP_OBJECT_CHANGED_UUID		"2ac8"

#define OTP_CP_CCC_DESC_UUID		"2902"

/* OTP Object Type Custom UUIDs */
/* SIG has not defined any UUID so far */
#define UNSUPPORTED_OBJECT_TYPE_UUID	"00007fb0-0000-1001-8000-00805f9b34fb"
#define FIRMWARE_UUID			"00007fb1-0000-1001-8000-00805f9b34fb"
#define ROUTE_GPX_UUID			"00007fb2-0000-1001-8000-00805f9b34fb"
#define TRACK_GPX_UUID			"00007fb3-0000-1001-8000-00805f9b34fb"

/* Object Metadata Length */
#define OTP_FEATURE_LENGTH		8
#define OTP_INDICATION_LEN_WITHOUT_RESP	3
#define OTP_INDICATION_LEN_WITH_RESP	7

/* Object ID range defined by SIG. */
#define OBJECT_START_ID	256
#define OBJECT_END_ID	281474976710655

/* Object Properties */
#define OBJECT_DELETE	0x00000001
#define OBJECT_EXECUTE	0x00000002
#define OBJECT_READ	0x00000004
#define OBJECT_WRITE	0x00000008
#define OBJECT_APPEND	0x00000010
#define OBJECT_TRUNCATE	0x00000020
#define OBJECT_PATCH	0x00000040
#define OBJECT_MARK	0x00000080

/* OACP opcodes */
#define OACP_CREATE		0x01
#define OACP_DELETE		0x02
#define OACP_CALC_CHECKSUM	0x03
#define OACP_EXECUTE		0x04
#define OACP_READ		0x05
#define OACP_WRITE		0x06
#define OACP_ABORT		0x07
#define OACP_RESPONSE		0x60

/* OACP error codes */
#define OACP_SUCCESS			0x01
#define OACP_OPCODE_NOT_SUPPORTED	0x02
#define OACP_INVALID_PARAM		0x03
#define OACP_INSUFFICIENT_RESOURCES	0x04
#define OACP_INVALID_OBJ		0x05
#define OACP_CHANNEL_UNAVAILABLE	0x06
#define OACP_UNSUPPORTED_TYPE		0x07
#define OACP_PROCEDURE_NOT_SUPPORTED	0x08
#define OACP_OBJECT_LOCKED		0x09
#define OACP_OPERATION_FAILED		0x0A

/* OLCP opcodes */
#define OLCP_FIRST		0x01
#define OLCP_LAST		0x02
#define OLCP_PREVIOUS		0x03
#define OLCP_NEXT		0x04
#define OLCP_GOTO		0x05
#define OLCP_ORDER		0x06
#define OLCP_REQ_NO_OBJ		0x07
#define OLCP_CLEAR_MARKING	0x08
#define OLCP_RESPONSE		0x70

/* OLCP order param values */
#define ORDER_BY_NAME_ASCENDING			0x01
#define ORDER_BY_TYPE_ASCENDING			0x02
#define ORDER_BY_CURR_SIZE_ASCENDING		0x03
#define ORDER_BY_FIRST_CREATED_ASCENDING	0x04
#define ORDER_BY_LAST_MODIFIED_ASCENDING	0x05
#define ORDER_BY_NAME_DESCENDING		0x11
#define ORDER_BY_TYPE_DESCENDING		0x12
#define ORDER_BY_CURR_SIZE_DESCENDING		0x13
#define ORDER_BY_FIRST_CREATED_DESCENDING	0x14
#define ORDER_BY_LAST_MODIFIED_DESCENDING	0x15

/* OLCP error codes */
#define OLCP_SUCCESS			0x01
#define OLCP_OPCODE_NOT_SUPPORTED	0x02
#define OLCP_INVALID_PARAM		0x03
#define OLCP_OPERATION_FAILED		0x04
#define OLCP_OUT_OF_BOUNDS		0x05
#define OLCP_TOO_MANY_OBJ		0x06
#define OLCP_NO_OBJ			0x07
#define OLCP_OJECT_ID_NOT_FOUND		0x08
