/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <dlog.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <net_connection.h>
#include <dbus/dbus.h>
#include <glib.h>
#include <dlog.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <net_connection.h>
#include <bundle.h>
#include <eventsystem.h>
#include <arpa/inet.h>
#include <vconf.h>

#include "bluetooth-api.h"
#include "bt-service-common.h"

#include <oal-manager.h>

#ifdef TIZEN_FEATURE_BT_IPSP
static GDBusProxy *ipsp_proxy;
#endif
static GDBusConnection *system_conn;
static GDBusConnection *session_conn;
static GDBusProxy *manager_proxy_g;
static GDBusProxy *adapter_proxy_g;
static void *net_conn;

static GDBusProxy *adapter_properties_proxy;

static GDBusConnection *system_gconn = NULL;

GDBusConnection *_bt_gdbus_init_system_gconn(void)
{
	GError *error = NULL;

	dbus_threads_init_default();

	if (system_gconn != NULL)
		return system_gconn;

	system_gconn = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &error);

	if (!system_gconn) {
		BT_ERR("Unable to connect to dbus: %s", error->message);
		g_clear_error(&error);
	}

	return system_gconn;
}

GDBusConnection *_bt_gdbus_get_system_gconn(void)
{
	GDBusConnection *local_system_gconn = NULL;
	GError *error = NULL;

	if (system_gconn == NULL) {
		system_gconn = _bt_gdbus_init_system_gconn();
	} else if (g_dbus_connection_is_closed(system_gconn)) {

		local_system_gconn = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &error);

		if (!local_system_gconn) {
			BT_ERR("Unable to connect to dbus: %s", error->message);
			g_clear_error(&error);
		}

		system_gconn = local_system_gconn;
	}

	return system_gconn;
}

static GDBusProxy *__bt_init_manager_proxy(void)
{
	GDBusProxy *proxy;

	if (system_conn == NULL) {
		system_conn = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, NULL);
		retv_if(system_conn == NULL, NULL);
	}

	proxy = g_dbus_proxy_new_sync(system_conn, G_DBUS_PROXY_FLAGS_NONE,
								NULL, BT_BLUEZ_NAME,
								BT_MANAGER_PATH, BT_MANAGER_INTERFACE,  NULL, NULL);

	retv_if(proxy == NULL, NULL);

	manager_proxy_g = proxy;

	return proxy;
}

static GDBusProxy *__bt_init_adapter_proxy(void)
{
	GDBusProxy *manager_proxy;
	GDBusProxy *proxy;
	char *adapter_path = NULL;

	if (system_conn == NULL) {
		system_conn = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, NULL);
		retv_if(system_conn == NULL, NULL);
	}

	manager_proxy = _bt_get_manager_proxy();
	retv_if(manager_proxy == NULL, NULL);

	adapter_path = _bt_get_adapter_path();
	retv_if(adapter_path == NULL, NULL);

	proxy = g_dbus_proxy_new_sync(system_conn, G_DBUS_PROXY_FLAGS_NONE,
								NULL, BT_BLUEZ_NAME,
								adapter_path, BT_ADAPTER_INTERFACE,  NULL, NULL);

	g_free(adapter_path);

	retv_if(proxy == NULL, NULL);

	adapter_proxy_g = proxy;

	return proxy;
}
#ifdef TIZEN_FEATURE_BT_IPSP
static GDBusProxy *__bt_init_ipsp_proxy(void)
{
	BT_DBG("+");

	GDBusConnection *g_conn;
	GDBusProxy *proxy;
	g_conn = _bt_gdbus_get_system_gconn();
	retv_if(g_conn == NULL, NULL);
	proxy = g_dbus_proxy_new_sync(g_conn, G_DBUS_PROXY_FLAGS_NONE,
								NULL, BT_IPSP_NAME,
								"/org/projectx/bt_ipsp", BT_IPSP_INTERFACE,  NULL, NULL);
	if (!proxy) {
		BT_ERR("Unable to get proxy");
		return NULL;
	}
	ipsp_proxy = proxy;
	BT_DBG("-");
	return proxy;
}
#endif
static GDBusProxy *__bt_init_adapter_properties_proxy(void)
{
	GDBusProxy *manager_proxy;
	GDBusProxy *proxy;
	char *adapter_path = NULL;

	if (system_conn == NULL) {
		system_conn = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, NULL);
		retv_if(system_conn == NULL, NULL);
	}

	manager_proxy = _bt_get_manager_proxy();
	retv_if(manager_proxy == NULL, NULL);

	adapter_path = _bt_get_adapter_path();
	retv_if(adapter_path == NULL, NULL);

	proxy = g_dbus_proxy_new_sync(system_conn, G_DBUS_PROXY_FLAGS_NONE,
									NULL, BT_BLUEZ_NAME,
									adapter_path, BT_PROPERTIES_INTERFACE,  NULL, NULL);

	g_free(adapter_path);

	retv_if(proxy == NULL, NULL);

	adapter_properties_proxy = proxy;

	return proxy;
}

GDBusConnection *__bt_init_system_gconn(void)
{
	if (system_conn == NULL)
		system_conn = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, NULL);

	return system_conn;
}

GDBusConnection *__bt_init_session_conn(void)
{
	if (session_conn == NULL)
		session_conn = g_bus_get_sync(G_BUS_TYPE_SESSION, NULL, NULL);

	return session_conn;
}

GDBusConnection *_bt_get_session_gconn(void)
{
	return (session_conn) ? session_conn : __bt_init_session_conn();
}

GDBusConnection *_bt_get_system_gconn(void)
{
	return (system_conn) ? system_conn : __bt_init_system_gconn();
}

GDBusConnection *_bt_get_system_conn(void)
{
	GDBusConnection *g_conn;

	if (system_conn == NULL)
		g_conn = __bt_init_system_gconn();
	else
		g_conn = system_conn;

	retv_if(g_conn == NULL, NULL);

	return g_conn;
}

GDBusProxy *_bt_get_manager_proxy(void)
{
	if (manager_proxy_g) {
		const gchar *path =  g_dbus_proxy_get_object_path(manager_proxy_g);
		if (path == NULL) {
			BT_ERR("Already proxy released hence creating new proxy");
			return  __bt_init_manager_proxy();
		}
		return manager_proxy_g;
	}
	return  __bt_init_manager_proxy();
}
#ifdef TIZEN_FEATURE_BT_IPSP
GDBusProxy *_bt_get_ipsp_proxy(void)
{
	if (ipsp_proxy) {
		const char *path =  g_dbus_proxy_get_object_path(ipsp_proxy);
		if (path == NULL) {
			BT_ERR("Already proxy released hence creating new proxy");
			return  __bt_init_ipsp_proxy();
		}
		return ipsp_proxy;
	}
	return  __bt_init_ipsp_proxy();
}
#endif

static void *__bt_init_net_conn(void)
{
	int result;
	connection_h connection = NULL;

	if (net_conn == NULL) {
		result = connection_create(&connection);

	if (result != CONNECTION_ERROR_NONE ||
					connection == NULL) {
		BT_DBG("connection_create() failed: %d", result);
		net_conn = NULL;
		return NULL;
	}
		net_conn = connection;
	}
	return net_conn;
}

void *_bt_get_net_conn(void)
{
	return (net_conn) ? net_conn : __bt_init_net_conn();
}

GDBusProxy *_bt_get_adapter_proxy(void)
{
	if (adapter_proxy_g) {
		const char *path =  g_dbus_proxy_get_object_path(adapter_proxy_g);
		if (path == NULL) {
			BT_ERR("Already proxy released hence creating new proxy");
			return  __bt_init_adapter_proxy();
		}

		return adapter_proxy_g;
	}
	return  __bt_init_adapter_proxy();

}

GDBusProxy *_bt_get_adapter_properties_proxy(void)
{
	return (adapter_properties_proxy) ? adapter_properties_proxy :
					__bt_init_adapter_properties_proxy();
}

static char *__bt_extract_adapter_path(GVariantIter *iter)
{
	char *object_path = NULL;
	GVariantIter *interface_iter;
	GVariantIter *svc_iter;
	char *interface_str = NULL;

	/* Parse the signature: oa{sa{sv}}} */
	while (g_variant_iter_loop(iter, "{&oa{sa{sv}}}", &object_path,
			&interface_iter)) {

		if (object_path == NULL)
			continue;

		while (g_variant_iter_loop(interface_iter, "{&sa{sv}}",
				&interface_str, &svc_iter)) {
			if (g_strcmp0(interface_str, "org.bluez.Adapter1") != 0)
				continue;

			BT_DBG("Object Path: %s", object_path);
			g_variant_iter_free(svc_iter);
			g_variant_iter_free(interface_iter);
			return g_strdup(object_path);
		}
	}
	return NULL;
}

char *_bt_get_adapter_path(void)
{
	GDBusConnection *conn;
	GDBusProxy *manager_proxy;
	GVariant *result = NULL;
	GVariantIter *iter = NULL;
	char *adapter_path = NULL;

	conn = _bt_get_system_conn();
	retv_if(conn == NULL, NULL);

	manager_proxy = _bt_get_manager_proxy();
	retv_if(manager_proxy == NULL, NULL);

	result = g_dbus_proxy_call_sync(manager_proxy, "GetManagedObjects",
				NULL,
				G_DBUS_CALL_FLAGS_NONE,
				-1,
				NULL,
				NULL);
	if (!result) {
		BT_ERR("Can't get managed objects");
		return NULL;
	}

	/* signature of GetManagedObjects:  a{oa{sa{sv}}} */
	g_variant_get(result, "(a{oa{sa{sv}}})", &iter);

	adapter_path = __bt_extract_adapter_path(iter);
	g_variant_iter_free(iter);
	g_variant_unref(result);
	return adapter_path;
}

void _bt_deinit_bluez_proxy(void)
{
	if (manager_proxy_g) {
		g_object_unref(manager_proxy_g);
		manager_proxy_g = NULL;
	}

	if (adapter_proxy_g) {
		g_object_unref(adapter_proxy_g);
		adapter_proxy_g = NULL;
	}
	if (adapter_properties_proxy) {
		g_object_unref(adapter_properties_proxy);
		adapter_properties_proxy = NULL;
	}
}

void _bt_deinit_proxys(void)
{
	int ret;
	_bt_deinit_bluez_proxy();

	if (system_conn) {
		g_object_unref(system_conn);
		system_conn = NULL;
	}

	if (session_conn) {
		g_object_unref(session_conn);
		session_conn = NULL;
	}

	if (net_conn) {
		ret = connection_destroy(net_conn);
		net_conn = NULL;
		if (ret != 0)
			BT_ERR("connection_destroy failed : %d", ret);
	}
}

void _bt_convert_device_path_to_address(const char *device_path,
						char *device_address)
{
	char address[BT_ADDRESS_STRING_SIZE] = { 0 };
	char *dev_addr;

	ret_if(device_path == NULL);
	ret_if(device_address == NULL);

	dev_addr = strstr(device_path, "dev_");
	if (dev_addr != NULL) {
		char *pos = NULL;
		dev_addr += 4;
		g_strlcpy(address, dev_addr, sizeof(address));

		while ((pos = strchr(address, '_')) != NULL)
			*pos = ':';

		g_strlcpy(device_address, address, BT_ADDRESS_STRING_SIZE);
	}
}


void _bt_convert_addr_string_to_type(unsigned char *addr,
					const char *address)
{
	int i;
	char *ptr = NULL;

	ret_if(address == NULL);
	ret_if(addr == NULL);

	for (i = 0; i < BT_ADDRESS_LENGTH_MAX; i++) {
		addr[i] = strtol(address, &ptr, 16);
		if (ptr[0] != '\0') {
			if (ptr[0] != ':')
				return;

			address = ptr + 1;
		}
	}
}

void _bt_convert_addr_type_to_string(char *address,
				unsigned char *addr)
{
	ret_if(address == NULL);
	ret_if(addr == NULL);

	snprintf(address, BT_ADDRESS_STRING_SIZE,
			"%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X",
			addr[0], addr[1], addr[2],
			addr[3], addr[4], addr[5]);
}

gboolean _bt_compare_adddress(const bluetooth_device_address_t *addr1,
		const bluetooth_device_address_t *addr2)
{
	if (memcmp(&addr1->addr, &addr2->addr, 6) == 0)
		return TRUE;
	else
		return FALSE;
}

void _bt_print_device_address_t(const bluetooth_device_address_t *addr)
{
	BT_INFO("%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X\n",
		addr->addr[0], addr->addr[1], addr->addr[2],
		addr->addr[3], addr->addr[4], addr->addr[5]);
}

void _bt_divide_device_class(bluetooth_device_class_t *device_class,
				unsigned int cod)
{
	ret_if(device_class == NULL);

	device_class->major_class = (unsigned short)(cod & 0x00001F00) >> 8;
	device_class->minor_class = (unsigned short)((cod & 0x000000FC));
	device_class->service_class = (unsigned long)((cod & 0x00FF0000));

	if (cod & 0x002000) {
		device_class->service_class |=
		BLUETOOTH_DEVICE_SERVICE_CLASS_LIMITED_DISCOVERABLE_MODE;
	}
}

void _bt_free_le_device_info(bt_remote_le_dev_info_t *le_dev_info)
{
	ret_if(le_dev_info == NULL);

	g_free(le_dev_info->adv_data);
	g_free(le_dev_info);
}

int _bt_copy_utf8_string(char *dest, const char *src, unsigned int length)
{
	unsigned int i;
	const char *p = src;
	char *next;
	int count;

	if (dest == NULL || src == NULL)
		return BLUETOOTH_ERROR_INVALID_PARAM;

	BT_DBG("+src : %s", src);
	BT_DBG("+dest : %s", dest);

	i = 0;
	while (*p != '\0' && i < length) {
		next = g_utf8_next_char(p);
		count = next - p;

		while (count > 0 && ((i + count) < length)) {
			dest[i++] = *p;
			p++;
			count--;
		}
		p = next;
	}
	return BLUETOOTH_ERROR_NONE;
}

gboolean _bt_utf8_validate(char *name)
{
	BT_DBG("+");
	gunichar2 *u16;
	glong items_written = 0;

	if (FALSE == g_utf8_validate(name, -1, NULL))
		return FALSE;

	u16 = g_utf8_to_utf16(name, -1, NULL, &items_written, NULL);
	if (u16 == NULL)
		return FALSE;

	g_free(u16);

	if (items_written != g_utf8_strlen(name, -1))
		return FALSE;

	BT_DBG("-");
	return TRUE;
}

int _bt_register_osp_server_in_agent(int type, char *uuid, char *path, int fd)
{
	BT_DBG("+");

#if TODO_40 /* Need to add this function */
	if (!_bt_agent_register_osp_server(type, uuid, path, fd))
		return BLUETOOTH_ERROR_INTERNAL;
#endif
	return BLUETOOTH_ERROR_NONE;
}

int _bt_unregister_osp_server_in_agent(int type, char *uuid)
{
	BT_DBG("+");

#if TODO_40 /* Need to add this function */
	if (!_bt_agent_unregister_osp_server(type, uuid))
		return BLUETOOTH_ERROR_INTERNAL;
#endif
	return BLUETOOTH_ERROR_NONE;
}

int _bt_set_socket_non_blocking(int socket_fd)
{
	/* Set Nonblocking */
	long arg;

	arg = fcntl(socket_fd, F_GETFL);

	if (arg < 0)
		return -errno;

	if (arg & O_NONBLOCK)
		BT_ERR("Already Non-blocking \n");

	arg |= O_NONBLOCK;

	if (fcntl(socket_fd, F_SETFL, arg) < 0)
		return -errno;

	return BLUETOOTH_ERROR_NONE;
}

int _bt_set_non_blocking_tty(int sk)
{
	struct termios ti = {0,};
	int err;

	err = _bt_set_socket_non_blocking(sk);

	if (err < 0) {
		BT_ERR("Error in set non blocking!\n");
		return err;
	}

	tcflush(sk, TCIOFLUSH);

	/* Switch tty to RAW mode */
	cfmakeraw(&ti);
	tcsetattr(sk, TCSANOW, &ti);

	return BLUETOOTH_ERROR_NONE;
}

static char *__bt_extract_device_path(GVariantIter *iter, char *address)
{
	char *object_path = NULL;
	char device_address[BT_ADDRESS_STRING_SIZE] = { 0 };

	/* Parse the signature: oa{sa{sv}}} */
	while (g_variant_iter_loop(iter, "{&oa{sa{sv}}}", &object_path,
			NULL)) {
		retv_if(object_path == NULL, NULL);
		_bt_convert_device_path_to_address(object_path, device_address);
		if (g_strcmp0(address, device_address) == 0)
			return g_strdup(object_path);
	}
	return NULL;
}

char *_bt_get_device_object_path(char *address)
{
	char *object_path = NULL;
	GDBusConnection *conn;
	GDBusProxy *manager_proxy;
	GVariant *result = NULL;
	GVariantIter *iter = NULL;

	conn = _bt_get_system_conn();
	retv_if(conn == NULL, NULL);

	manager_proxy = _bt_get_manager_proxy();
	retv_if(manager_proxy == NULL, NULL);

	result = g_dbus_proxy_call_sync(manager_proxy, "GetManagedObjects",
				NULL,
				G_DBUS_CALL_FLAGS_NONE,
				-1,
				NULL,
				NULL);
	if (!result) {
		BT_ERR("Can't get managed objects");
		return NULL;
	}

	/* signature of GetManagedObjects:  a{oa{sa{sv}}} */
	g_variant_get(result, "(a{oa{sa{sv}}})", &iter);
	object_path = __bt_extract_device_path(iter, address);
	g_variant_iter_free(iter);
	g_variant_unref(result);
	return object_path;
}

char *_bt_get_profile_uuid128(bt_profile_type_t profile_type)
{
	switch (profile_type) {
	case BT_PROFILE_CONN_RFCOMM:
		return strdup(RFCOMM_UUID_STR);
	case BT_PROFILE_CONN_A2DP:
		return strdup(A2DP_SINK_UUID);
	case BT_PROFILE_CONN_A2DP_SINK:
		return strdup(A2DP_SOURCE_UUID);
	case BT_PROFILE_CONN_HSP:
		return strdup(HFP_HS_UUID);
	case BT_PROFILE_CONN_HID:
		return strdup(HID_UUID);
	case BT_PROFILE_CONN_NAP:
		return strdup(NAP_UUID);
	case BT_PROFILE_CONN_HFG:
		return strdup(HFP_AG_UUID);
	case BT_PROFILE_CONN_GATT:
	case BT_PROFILE_CONN_ALL: /* NULL UUID will connect to both the audio profiles*/
	default:
		return NULL;
	};
}

const char *_bt_convert_error_to_string(int error)
{
	switch (error) {
	case BLUETOOTH_ERROR_CANCEL:
		return "CANCELLED";
	case BLUETOOTH_ERROR_INVALID_PARAM:
		return "INVALID_PARAMETER";
	case BLUETOOTH_ERROR_INVALID_DATA:
		return "INVALID DATA";
	case BLUETOOTH_ERROR_MEMORY_ALLOCATION:
	case BLUETOOTH_ERROR_OUT_OF_MEMORY:
		return "OUT_OF_MEMORY";
	case BLUETOOTH_ERROR_TIMEOUT:
		return "TIMEOUT";
	case BLUETOOTH_ERROR_NO_RESOURCES:
		return "NO_RESOURCES";
	case BLUETOOTH_ERROR_INTERNAL:
		return "INTERNAL";
	case BLUETOOTH_ERROR_NOT_SUPPORT:
		return "NOT_SUPPORT";
	case BLUETOOTH_ERROR_DEVICE_NOT_ENABLED:
		return "NOT_ENABLED";
	case BLUETOOTH_ERROR_DEVICE_ALREADY_ENABLED:
		return "ALREADY_ENABLED";
	case BLUETOOTH_ERROR_DEVICE_BUSY:
		return "DEVICE_BUSY";
	case BLUETOOTH_ERROR_ACCESS_DENIED:
		return "ACCESS_DENIED";
	case BLUETOOTH_ERROR_MAX_CLIENT:
		return "MAX_CLIENT";
	case BLUETOOTH_ERROR_NOT_FOUND:
		return "NOT_FOUND";
	case BLUETOOTH_ERROR_SERVICE_SEARCH_ERROR:
		return "SERVICE_SEARCH_ERROR";
	case BLUETOOTH_ERROR_PARING_FAILED:
		return "PARING_FAILED";
	case BLUETOOTH_ERROR_NOT_PAIRED:
		return "NOT_PAIRED";
	case BLUETOOTH_ERROR_SERVICE_NOT_FOUND:
		return "SERVICE_NOT_FOUND";
	case BLUETOOTH_ERROR_NOT_CONNECTED:
		return "NOT_CONNECTED";
	case BLUETOOTH_ERROR_ALREADY_CONNECT:
		return "ALREADY_CONNECT";
	case BLUETOOTH_ERROR_CONNECTION_BUSY:
		return "CONNECTION_BUSY";
	case BLUETOOTH_ERROR_CONNECTION_ERROR:
		return "CONNECTION_ERROR";
	case BLUETOOTH_ERROR_MAX_CONNECTION:
		return "MAX_CONNECTION";
	case BLUETOOTH_ERROR_NOT_IN_OPERATION:
		return "NOT_IN_OPERATION";
	case BLUETOOTH_ERROR_CANCEL_BY_USER:
		return "CANCEL_BY_USER";
	case BLUETOOTH_ERROR_REGISTRATION_FAILED:
		return "REGISTRATION_FAILED";
	case BLUETOOTH_ERROR_IN_PROGRESS:
		return "IN_PROGRESS";
	case BLUETOOTH_ERROR_AUTHENTICATION_FAILED:
		return "AUTHENTICATION_FAILED";
	case BLUETOOTH_ERROR_HOST_DOWN:
		return "HOST_DOWN";
	case BLUETOOTH_ERROR_END_OF_DEVICE_LIST:
		return "END_OF_DEVICE_LIST";
	case BLUETOOTH_ERROR_AGENT_ALREADY_EXIST:
		return "AGENT_ALREADY_EXIST";
	case BLUETOOTH_ERROR_AGENT_DOES_NOT_EXIST:
		return "AGENT_DOES_NOT_EXIST";
	case BLUETOOTH_ERROR_ALREADY_INITIALIZED:
		return "ALREADY_INITIALIZED";
	case BLUETOOTH_ERROR_PERMISSION_DEINED:
		return "PERMISSION_DEINED";
	case BLUETOOTH_ERROR_ALREADY_DEACTIVATED:
		return "ALREADY_DEACTIVATED";
	case BLUETOOTH_ERROR_NOT_INITIALIZED:
		return "NOT_INITIALIZED";
	default:
		return "UNKNOWN";
	}
}

const char * _bt_convert_disc_reason_to_string(int reason)
{
	switch (reason) {
	case 1:
		return "Link loss";
	case 2:
		return "Connection terminated by local host";
	case 3:
		return "Remote user terminated connection";
	case 0:
	default:
		return "Unknown";
	}
}

void _bt_logging_connection(gboolean connect, int addr_type)
{
	static int le_conn = 0;
	static int le_disc = 0;
	static int edr_conn = 0;
	static int edr_disc = 0;

	if (connect) {
		if (addr_type)
			le_conn++;
		else
			edr_conn++;
	} else {
		if (addr_type)
			le_disc++;
		else
			edr_disc++;
	}

	BT_INFO("[PM] Number of LE conn: %d disc: %d, Number of BR/EDR conn: %d disc: %d",
			le_conn, le_disc, edr_conn, edr_disc);
}

int _bt_eventsystem_set_value(const char *event, const char *key, const char *value)
{
	int ret;
	bundle *b = NULL;

	b = bundle_create();

	bundle_add_str(b, key, value);

	ret = eventsystem_send_system_event(event, b);

	BT_DBG("eventsystem_send_system_event result: %d", ret);

	bundle_free(b);

	return ret;
}

void _bt_swap_byte_ordering(char *data, int data_len)
{
	char temp;
	int i, j;

	ret_if(data == NULL);
	/* Swap to opposite endian */
	for (i = 0, j = data_len - 1; i < data_len; i++, j--) {
		temp = data[i];
		data[i] = data[j];
		data[j] = temp;
		}
}

int _bt_byte_arr_cmp(const char *data1, const char *data2, int data_len)
{
	int i;

	retv_if(data1 == NULL, -1);
	retv_if(data2 == NULL, -1);
	for (i = 0; i < data_len; i++) {
		if (data1[i] != data2[i])
			return data1[i] - data2[i];
		}
	return 0;
}
int _bt_byte_arr_cmp_with_mask(const char *data1, const char *data2,
	const char *mask, int data_len)
{
	int i;
	char a, b;

	retv_if(data1 == NULL, -1);
	retv_if(data2 == NULL, -1);
	retv_if(mask == NULL, -1);
	for (i = 0; i < data_len; i++) {
		a = data1[i] & mask[i];
		b = data2[i] & mask[i];
		if (a != b)
			return (int)(a - b);
		}
	return 0;
}

void _bt_copy_remote_dev(bt_remote_dev_info_t *dev_info, remote_device_t *oal_device)
{
	unsigned int i;

	BT_DBG("+");

	dev_info->address = g_new0(char, BT_ADDRESS_STRING_SIZE);
	_bt_convert_addr_type_to_string(dev_info->address, oal_device->address.addr);
	BT_INFO("Address [%s]", dev_info->address);

	if (strlen(oal_device->name) == 0)
		dev_info->name = NULL;
	else {
		dev_info->name = g_strdup(oal_device->name);
		_bt_truncate_non_utf8_chars(dev_info->name);
		BT_INFO("Name [%s]", dev_info->name);
	}

	if (strlen(oal_device->alias) == 0) {
		dev_info->alias = NULL;
	} else {
		dev_info->alias = g_strdup(oal_device->alias);
		_bt_truncate_non_utf8_chars(dev_info->alias);
		BT_INFO("Alias [%s]", dev_info->alias);
	}

	dev_info->class = oal_device->cod;
	dev_info->paired = oal_device->is_bonded;
	BT_INFO("Is Bonded [%d]", dev_info->paired);
	dev_info->connected = oal_device->is_connected;
	BT_INFO("iS Connected [%d]", dev_info->connected);
	dev_info->rssi = oal_device->rssi;
	dev_info->addr_type = oal_device->type;
	dev_info->uuid_count = oal_device->uuid_count;
	BT_INFO("UUID Count [%d]", dev_info->uuid_count);
	dev_info->trust = ((oal_device->is_trusted == 0) ? FALSE : TRUE);

	if (dev_info->uuid_count > 0)
		dev_info->uuids = g_new0(char *, dev_info->uuid_count);

	/* Fill Remote Device Service List list */
	for (i = 0; i < dev_info->uuid_count; i++) {
		dev_info->uuids[i] = g_malloc0(BLUETOOTH_UUID_STRING_MAX);
		_bt_uuid_to_string((service_uuid_t *)&oal_device->uuid[i].uuid, dev_info->uuids[i]);
	}

	dev_info->vid = (unsigned short)oal_device->vid;
	dev_info->pid = (unsigned short)oal_device->pid;
	dev_info->device_type = (int)oal_device->type;
	dev_info->is_alias_set = ((oal_device->is_alias_set == 0) ? FALSE : TRUE);

	BT_DBG("-");
}

void _bt_free_remote_dev(bt_remote_dev_info_t *dev_info)
{
	unsigned int c;

	ret_if(NULL == dev_info);

	BT_INFO("+");

	g_free(dev_info->address);
	g_free(dev_info->name);
	g_free(dev_info->alias);
	g_free(dev_info->manufacturer_data);

	for (c = 0; c < dev_info->uuid_count; c++)
		g_free(dev_info->uuids[c]);
	g_free(dev_info->uuids);
	g_free(dev_info);

	BT_INFO("-");
}

static void __bt_get_service_list(bt_remote_dev_info_t *info, bluetooth_device_info_t *dev)
{
	unsigned int i;
	char **uuids;
	char **parts;

	BT_DBG("+");

	ret_if(info == NULL);
	ret_if(dev == NULL);

	uuids = info->uuids;
	if (uuids == NULL) {
		BT_ERR("No UUID's");
		return;
	}

	dev->service_index = 0;
	BT_DBG("Total UUID count [%d]", info->uuid_count);
	for (i = 0; i < info->uuid_count; i++) {
		g_strlcpy(dev->uuids[i], uuids[i], BLUETOOTH_UUID_STRING_MAX);

		parts = g_strsplit(uuids[i], "-", -1);

		if (parts == NULL || parts[0] == NULL)
			break;

		dev->service_list_array[i] = g_ascii_strtoull(parts[0], NULL, 16);
		g_strfreev(parts);

		dev->service_index++;
	}

	BT_DBG("-");
}

void _bt_copy_remote_device(bt_remote_dev_info_t *rem_dev, bluetooth_device_info_t *dev)
{
	BT_DBG("+");

	memset(dev, 0x00, sizeof(bluetooth_device_info_t));
	__bt_get_service_list(rem_dev, dev);
	_bt_convert_addr_string_to_type(dev->device_address.addr, rem_dev->address);
	_bt_divide_device_class(&dev->device_class, rem_dev->class);

	if (rem_dev->alias) {
		g_strlcpy(dev->device_name.name, rem_dev->alias,
				BLUETOOTH_DEVICE_NAME_LENGTH_MAX + 1);
		BT_DBG("Name: %s", dev->device_name.name);
	} else {
		g_strlcpy(dev->device_name.name, rem_dev->name,
				BLUETOOTH_DEVICE_NAME_LENGTH_MAX + 1);
		BT_DBG("Name: %s", dev->device_name.name);
	}

	dev->rssi = rem_dev->rssi;
	dev->trust = rem_dev->trust;
	dev->paired = rem_dev->paired;
	dev->connected = rem_dev->connected;
	dev->is_alias_set = rem_dev->is_alias_set;

	/* Fill Manufacturer data */
	if (rem_dev->manufacturer_data_len > 0) {
		dev->manufacturer_data.data_len = rem_dev->manufacturer_data_len;
		memcpy(dev->manufacturer_data.data,
			rem_dev->manufacturer_data, rem_dev->manufacturer_data_len);
	} else {
		dev->manufacturer_data.data_len = 0;
	}

	BT_DBG("-");
}

void _bt_service_print_dev_info(bluetooth_device_info_t *dev_info)
{
	int i;

	ret_if(dev_info == NULL);

	_bt_print_device_address_t(&(dev_info->device_address));
	BT_INFO("Device Name:[%s]", dev_info->device_name.name);
	BT_INFO("Device Major Class:[0x%X]", dev_info->device_class.major_class);
	BT_INFO("Device Minor Class:[0x%X]", dev_info->device_class.minor_class);
	BT_INFO("Device Service Class:[0x%X]", dev_info->device_class.minor_class);
	BT_INFO("Device Paired:[%s]", (dev_info->paired ? "TRUE" : "FALSE"));
	BT_INFO("Device Trusted:[%s]", (dev_info->trust ? "TRUE" : "FALSE"));
	BT_INFO("Device Connected:[%d]", dev_info->connected);
	BT_INFO("Device Service index:[%d]", dev_info->service_index);
	for (i = 0; i < dev_info->service_index; i++) {
		BT_INFO("Device Service List:[%d]", dev_info->service_list_array[i]);
		BT_INFO("Device UUID:[%s]", dev_info->uuids[i]);
	}

	BT_INFO("Device manufacturer data len:[%d]", dev_info->manufacturer_data.data_len);
	for (i = 0; i < dev_info->manufacturer_data.data_len; i++)
		BT_INFO("%2.2X", dev_info->manufacturer_data.data[i]);
}

void _bt_uuid_to_string(service_uuid_t *p_uuid, char *str)
{
	uint32_t uuid0, uuid4;
	uint16_t uuid1, uuid2, uuid3, uuid5;

	memcpy(&uuid0, &(p_uuid->uuid[0]), 4);
	memcpy(&uuid1, &(p_uuid->uuid[4]), 2);
	memcpy(&uuid2, &(p_uuid->uuid[6]), 2);
	memcpy(&uuid3, &(p_uuid->uuid[8]), 2);
	memcpy(&uuid4, &(p_uuid->uuid[10]), 4);
	memcpy(&uuid5, &(p_uuid->uuid[14]), 2);

	snprintf((char *)str, BLUETOOTH_UUID_STRING_MAX, "%.8x-%.4x-%.4x-%.4x-%.8x%.4x",
			ntohl(uuid0), ntohs(uuid1),
			ntohs(uuid2), ntohs(uuid3),
			ntohl(uuid4), ntohs(uuid5));
	return;
}

void _bt_string_to_uuid(char *str, service_uuid_t *p_uuid)
{
	uint32_t uuid0, uuid4;
	uint16_t uuid1, uuid2, uuid3, uuid5;

	sscanf(str, "%08x-%04hx-%04hx-%04hx-%08x%04hx",
			&uuid0, &uuid1, &uuid2, &uuid3, &uuid4, &uuid5);

	uuid0 = htonl(uuid0);
	uuid1 = htons(uuid1);
	uuid2 = htons(uuid2);
	uuid3 = htons(uuid3);
	uuid4 = htonl(uuid4);
	uuid5 = htons(uuid5);

	memcpy(&(p_uuid->uuid[0]), &uuid0, 4);
	memcpy(&(p_uuid->uuid[4]), &uuid1, 2);
	memcpy(&(p_uuid->uuid[6]), &uuid2, 2);
	memcpy(&(p_uuid->uuid[8]), &uuid3, 2);
	memcpy(&(p_uuid->uuid[10]), &uuid4, 4);
	memcpy(&(p_uuid->uuid[14]), &uuid5, 2);

	return;
}

void _bt_convert_byte_ordering(char *data, int data_len,
		char *converted_data)
{
	int i, j;
	char temp[BT_UUID_LEN];

	memcpy(&temp, data, data_len);
	for (i = 0, j = data_len - 1; i < data_len; i++, j--)
		converted_data[i] = temp[j];

}

/* Trim string at first non-utf8 char */
void _bt_truncate_non_utf8_chars(char * str)
{
	guint i = 0;
	const char *ptr = NULL;

	if (strlen(str) != 0) {
		if (!g_utf8_validate(str, -1, &ptr)) {
			while (*(str + i) != *ptr)
				i++;
			*(str + i) = '\0';
		}
	}
}

invocation_info_t* _bt_get_request_info_data(int service_function, char *address)
{
	GSList *l;
	invocation_info_t *req_info = NULL;

	BT_DBG("+");

	retv_if(NULL == address, NULL);

	/* Get method invocation context */
	for (l = _bt_get_invocation_list(); l != NULL; l = g_slist_next(l)) {
		req_info = l->data;
		if (req_info == NULL || req_info->service_function != service_function)
			continue;

		if (!strncasecmp((char *)req_info->user_data, address, BT_ADDRESS_STRING_SIZE))
			return req_info;
	}

	BT_DBG("-");
	return NULL;
}

#ifdef TIZEN_GATT_CLIENT
invocation_info_t* _bt_get_request_info_data_from_function_name(int service_function)
{
	GSList *l;
	invocation_info_t *req_info = NULL;

	BT_DBG("+");

	/* Get method invocation context */
	for (l = _bt_get_invocation_list(); l != NULL; l = g_slist_next(l)) {
		req_info = l->data;
		if (req_info == NULL || req_info->service_function != service_function)
			continue;
		return req_info;
	}

	BT_DBG("-");
	return NULL;
}
#endif

void _bt_set_device_values(gboolean connected, int state)
{
	int bt_device_state = VCONFKEY_BT_DEVICE_NONE;

	if (vconf_get_int(VCONFKEY_BT_DEVICE, &bt_device_state) != 0)
		BT_ERR("vconf_get_int failed");

	if (connected == TRUE)
		bt_device_state |= state;
	else if (bt_device_state & state)
		bt_device_state ^= state;

	if (vconf_set_int(VCONFKEY_BT_DEVICE, bt_device_state) != 0)
		BT_ERR("vconf_set_int failed");
}

int _bt_get_ad_data_by_type(char *in_data, int in_len,
		char in_type, char **data, int *data_len)
{
	if (in_data == NULL || data == NULL || data_len == NULL)
		return BLUETOOTH_ERROR_INVALID_PARAM;

	if (in_len < 0)
		return BLUETOOTH_ERROR_INVALID_PARAM;

	int i;
	int len = 0;
	int type = 0;

	for (i = 0; i < in_len; i++) {
		len = in_data[i];
		if (len <= 0 || i + 1 >= in_len) {
			BT_ERR("Invalid advertising data");
			return BLUETOOTH_ERROR_INVALID_PARAM;
		}

		type = in_data[i + 1];
		if (type == in_type) {
			i = i + 2;
			len--;
			break;
		}

		i += len;
		len = 0;
	}

	if (i + len > in_len) {
		BT_ERR("Invalid advertising data");
		return BLUETOOTH_ERROR_INVALID_PARAM;
	} else if (len == 0) {
		BT_INFO("AD Type 0x%02x data is not set", in_type);
		*data = NULL;
		*data_len = 0;
		return BLUETOOTH_ERROR_NONE;
	}

	*data = g_memdup(&in_data[i], len);
	if (*data == NULL)
		return BLUETOOTH_ERROR_OUT_OF_MEMORY;
	*data_len = len;

	return BLUETOOTH_ERROR_NONE;
}

void _bt_copy_remote_dev_info(bt_remote_dev_info_t *dest, bt_remote_dev_info_t *src)
{
	BT_DBG("+");

	ret_if(NULL == src);

	if (!dest->address) {
		dest->address = g_strdup(src->address);
		BT_DBG("Address: [%s]", dest->address);
	}

	if (src->name) {
		g_free(dest->name);
		dest->name = g_strdup(src->name);
		BT_DBG("Name: [%s]", dest->name);
	}

	if (src->alias) {
		g_free(dest->alias);
		dest->alias = g_strdup(src->alias);
		BT_DBG("Alias: [%s]", dest->alias);
	}

	if (src->uuid_count > 0 && src->uuids) {
		int i;

		/* Release previously allocated UUIDs */
		if (dest->uuid_count > 0 && dest->uuids)
			for (i = 0; i < dest->uuid_count; i++)
				g_free(dest->uuids[i]);
		g_free(dest->uuids);

		/* Copy new UUID list */
		dest->uuids = g_malloc0(src->uuid_count * sizeof(char *));
		for (i = 0; i < src->uuid_count; i++)
			dest->uuids[i] = g_strdup(src->uuids[i]);

		dest->uuid_count = src->uuid_count;
		BT_DBG("UUID count: %d", dest->uuid_count);
	}

	if (src->manufacturer_data_len > 0 && src->manufacturer_data) {
		BT_DBG("Manufacturer data length: %d", dest->manufacturer_data_len);
		if (dest->manufacturer_data)
			g_free(dest->manufacturer_data);

		dest->manufacturer_data_len = src->manufacturer_data_len;
		dest->manufacturer_data = g_memdup(
			src->manufacturer_data, src->manufacturer_data_len);
		BT_DBG("Manufacturer data length: %d", dest->manufacturer_data_len);
	}

	dest->rssi = src->rssi;
	dest->class = src->class;
	dest->paired = src->paired;
	dest->connected = src->connected;
	dest->trust = src->trust;
	dest->addr_type = src->addr_type;
	dest->vid = src->vid;
	dest->pid = src->pid;
	dest->device_type = src->device_type;
	dest->is_alias_set = src->is_alias_set;

	BT_DBG("-");
}
