/*
 * Bluetooth-frwk
 *
 * Copyright (c) 2015 - 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact:  Anupam Roy <anupam.r@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef __BT_SERVICE_AUDIO_COMMON_H__
#define __BT_SERVICE_AUDIO_COMMON_H__

#include "bluetooth-audio-api.h"
#include "bt-service-event-receiver.h"
#include  <bt-service-common.h>

#ifdef __cplusplus
extern "C" {
#endif
typedef enum {
	BT_PENDING_NONE = 0x00,
	BT_PENDING_CONNECT,
	BT_PENDING_DISCONNECT
} bt_pending_request_t;

typedef enum {
	BT_AUDIO_HSP = 0x01,	/* HSP Connection */
	BT_AUDIO_A2DP = 0x02,		/* A2DP Source Connection, remote device is A2DP Sink */
	BT_AUDIO_ALL = 0x03,		/* HSP and A2DP Source Connection (BT_AUDIO_HSP and BT_AUDIO_A2DP) */
	BT_AVRCP_TARGET = 0x04, /* AVRCP Target Connection to remote AVRCP controller */
	BT_AVRCP = 0x08,	/* AVRCP ctrl Connection to remote AVRCP target */
	BT_AUDIO_A2DP_SOURCE = 0x10,	/* A2DP Sink Connection, remote device is A2DP Source */
	BT_AUDIO_HFP_SOURCE = 0x11	/* HSP and A2DP Sink Connection (BT_AUDIO_HSP and BT_AUDIO_A2DP_SOURCE)*/
} bt_audio_type_t;

typedef enum {
	SERVICE_SEARCH_NOT_STARTED,
	SERVICE_SEARCH_STARTED,
	SERVICE_SEARCH_DONE,
} bt_audio_service_search_e;

typedef struct {
	char *address;
	bt_audio_type_t type;
	bt_audio_type_t conn_req_type;
	bt_audio_service_search_e search_status;
	gboolean is_a2dp_src_supported;
	gboolean is_a2dp_sink_supported;
	gboolean is_hfp_supported;
	unsigned char bonded_info_retries;
} bt_pending_audio_conn_t;

typedef struct {
	char *address;
	bt_pending_request_t  pending;
	int type;
} bt_audio_function_data_t;

typedef struct {
	int type;
	int disconnection_type;
	char *address;
	gboolean ag_flag;
} bt_headset_wait_t;

typedef enum {
	BT_STATE_NONE = 0x00,
	BT_STATE_CONNECTING,
	BT_STATE_CONNECTED,
	BT_STATE_DISCONNECTING,
	BT_STATE_DISCONNECTED
} bt_headset_device_state_t;

int _bt_audio_initialize(bt_service_module_t module);

int _bt_audio_connect(int type, bluetooth_device_address_t *device_address);

int _bt_audio_disconnect(int type, bluetooth_device_address_t *device_address);

gboolean _bt_is_service_connected(char* address, int type);

void _bt_remove_service_search_request(char *address);

void _bt_remove_headset_from_list(int type, const char *address);

bt_headset_wait_t *_bt_get_audio_wait_data(void);

void _bt_rel_wait_data(void);

void _bt_set_audio_wait_data_flag(gboolean flag);

void _bt_audio_check_pending_connect(char *address);

gboolean _bt_is_headset_type_connected(int type, char *address);

gboolean _bt_is_headset_address_type_connected(int type, const char *address);

void _bt_add_headset_to_list(int type, int status, const char *address);

void _bt_remove_service_search_request(char *address);

bt_pending_audio_conn_t* _bt_get_pending_audio_conn_info(char *address);

void _bt_cleanup_audio_pending_conn_info_and_reply_pending_req(bt_pending_audio_conn_t *info,
							int result);
gboolean _bt_audio_check_pending_connection(char *address);

void _bt_audio_check_pending_disconnection(char *address, int type);

int _bt_audio_select_role(bluetooth_audio_role_t role);

void _bt_audio_set_current_role(bluetooth_audio_role_t role);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BT_SERVICE_AUDIO_COMMON_H__ */
