/*
 * Bluetooth-frwk
 *
 * Copyright (c) 2015 - 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact:  Nilesh Trimbake <t.shripati@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __BT_SERVICE_AVRCP_CONTROLLER_H__
#define __BT_SERVICE_AVRCP_CONTROLLER_H__

#include "bt-service-event-receiver.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <oal-manager.h>

#include <bt-service-audio-common.h>
#include <oal-avrcp-ct.h>

typedef struct {
	const char *title;
	const char *artist;
	const char *album;
	const char *genre;
	unsigned int total_tracks;
	unsigned int number;
	int64_t duration;
} _bt_avrcp_ct_media_metadata_attr_t;

typedef struct {
	unsigned int value;
} _bt_avrcp_ct_property_value_t;

typedef enum {
	RC_PASS_CMD_PLAY = 0x01,
	RC_PASS_CMD_PAUSE,
	RC_PASS_CMD_STOP,
	RC_PASS_CMD_NEXT,
	RC_PASS_CMD_PREVIOUS,
	RC_PASS_CMD_PRESS_FAST_FORWARD,
	RC_PASS_CMD_RELEASE_FAST_FORWARD,
	RC_PASS_CMD_PRESS_REWIND,
	RC_PASS_CMD_RELEASE_REWIND,
	RC_PASS_CMD_VOLUME_UP,
	RC_PASS_CMD_VOLUME_DOWN
} _bt_avrcp_ct_pass_cmd_key_code_t;

int _bt_avrcp_connect_remote_target(bluetooth_device_address_t *address);

int _bt_avrcp_disconnect_remote_target(bluetooth_device_address_t *address);

int _bt_avrcp_control_cmd(int type);

int _bt_avrcp_control_cmd_to_dest(int type, bluetooth_device_address_t *address);

int _bt_avrcp_control_set_property(int type, unsigned int value);

int _bt_avrcp_transport_set_property(int type, unsigned int value);

int _bt_avrcp_control_get_property(int type);

int _bt_avrcp_control_get_track_info(void);

void _bt_set_control_device_path(const char *path);

void _bt_remove_control_device_path(const char *path);

void _bt_avrcp_ctrl_event_handler(int oal_event, gpointer event_data);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BT_SERVICE_AVRCP_CONTROLLER_H__ */
