/*
 * Copyright (c) 2015 - 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact:  Anupam Roy <anupam.r@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _BT_SERVICE_CORE_DEVICE_H_
#define _BT_SERVICE_CORE_DEVICE_H_

#include <sys/types.h>
#include "bluetooth-api.h"

#ifdef __cplusplus
extern "C" {
#endif

#define BT_DEVICE_PIN_CODE_SLOT_MAX 10
typedef struct {
	char *address;
	char *pin_code;
} bt_pin_code_info_t;

gboolean _bt_is_device_creating(void);

#ifdef TIZEN_GATT_CLIENT
int _bt_le_connection_update(const char *sender,
		unsigned char *device_address,
		float interval_min, float interval_max,
		guint16 latency, guint16 time_out);
#endif

#ifdef TIZEN_FEATURE_BT_OBEX
bluetooth_trusted_profile_t _bt_get_trusted_profile_enum(const char *uuid);
#endif

void _bt_device_state_handle_callback_set_request(void);

int _bt_device_get_bonded_device_info(bluetooth_device_address_t *addr);

int _bt_set_alias(bluetooth_device_address_t *device_address, const char *alias);

int _bt_bond_device(bluetooth_device_address_t *device_address,
		unsigned short conn_type, GArray **out_param1);

int _bt_unbond_device(bluetooth_device_address_t *device_address,
		GArray **out_param1);

int _bt_cancel_bonding(void);

gboolean _bt_is_bonding_device_address(const char *address);

gboolean _bt_device_is_bonding(void);

gboolean _bt_device_is_pairing(void);

int _bt_cancel_search_device(void);

int _bt_search_device(bluetooth_device_address_t *device_address);

gboolean _bt_is_bonding_device_address(const char *address);

void _bt_set_autopair_status_in_bonding_info(gboolean is_autopair);

int _bt_passkey_reply(const char *passkey, gboolean authentication_reply);

int _bt_passkey_confirmation_reply(gboolean confirmation_reply);

int _bt_set_authorization(bluetooth_device_address_t *device_address,
				gboolean authorize);

gboolean _bt_is_device_connected(bluetooth_device_address_t *device_address, int svc_type);

int _bt_rfcomm_reply_conn_authorization(char *address, gboolean reply);

int _bt_get_bonded_device_info(bluetooth_device_address_t *dev_addr);

int _bt_get_bonded_devices(void);

bt_remote_dev_info_t *_bt_service_get_remote_dev_info(const char *address);

gboolean _bt_is_bonded_devices_retrived(void);

void _bt_device_handle_paired_address_list(bluetooth_device_address_t *addr_list, int count);

void _bt_device_handle_adapter_state(gboolean enabled);

int _bt_is_alias_set(bluetooth_device_address_t *device_address);

int _bt_get_connected_link(bluetooth_device_address_t *device_address);

int _bt_set_trust_profile(bluetooth_device_address_t *bd_addr,
		bluetooth_trusted_profile_t profile, gboolean trust);

int _bt_get_trust_profile(bluetooth_device_address_t *bd_addr,
		bluetooth_trusted_profile_t profile, guint *trust);

int _bt_enable_rssi(bluetooth_device_address_t *addr, int link_type,
		int low_threshold, int in_range_threshold, int high_threshold);

int _bt_get_rssi_strength(bluetooth_device_address_t *bd_addr, int link_type);

int _bt_set_passkey_notification(const char *sender, gboolean enable);

int _bt_set_pin_code(bluetooth_device_address_t *device_address,
		bluetooth_device_pin_code_t *pin_code);

int _bt_unset_pin_code(bluetooth_device_address_t *device_address);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_BT_SERVICE_CORE_DEVICE_H_*/

