/*
 * Bluetooth-frwk
 *
 * Copyright (c) 2016-2017 Samsung Electronics Co., Ltd.
 *
 * Contact: Anupam Roy (anupam.r@samsung.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef BT_SERVICE_GATT_H_
#define BT_SERVICE_GATT_H_

#include <glib.h>
#include <sys/types.h>
#include "bluetooth-api.h"
#include "bluetooth-gatt-server-api.h"

#ifdef TIZEN_GATT_CLIENT
#include "bluetooth-gatt-client-api.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

int _bt_gatt_init(void);

void _bt_gatt_deinit(void);

int _bt_get_allocated_server_instance(const char *sender, int adv_handle, gboolean use_reserved_slot);

int _bt_register_server_instance(const char *sender, int adv_handle);

int _bt_unregister_server_instance(const char *sender, int adv_handle);

char* _bt_get_sender_and_handle(int server_instance, int *adv_handle);

int _bt_register_server_instance(const char *sender, int adv_handle);

void _bt_get_adv_handle_from_instance(int server_inst, int *adv_handle);

void _bt_set_new_scan_rsp_data(bluetooth_scan_resp_data_t *scan, int len, int instance);

void _bt_set_new_adv_data(bluetooth_advertising_data_t *adv, int len, int instance);

void _bt_update_adv_handle(const char *sender, int adv_handle);

void _bt_get_previous_adv_data(bluetooth_advertising_data_t *adv, int *len, int instance);

void _bt_get_previous_scan_rsp_data(bluetooth_scan_resp_data_t *scan, int *len, int instance);

int _bt_is_sender_gatt_server_with_no_adv(const char *sender, int adv_handle);

void _bt_check_adv_app_termination(const char *name);

int _bt_gatt_server_add_service(char *sender, int service_type,
		int num_handles, char *svc_uuid, int instance_id);

int _bt_gatt_server_add_included_service(char *sender, int instance_id,
		int service_handle, int included_svc_handle);

int _bt_gatt_server_add_characteristic(char *sender, char *char_uuid,
		bluetooth_gatt_server_attribute_params_t *param);

int _bt_gatt_server_add_descriptor(char *sender, char *desc_uuid,
		bt_gatt_permission_t *perm, int service_handle, int instance_id);

int _bt_gatt_server_start_service(char *sender, int service_handle, int instance_id);

int _bt_gatt_server_stop_service(char *sender, int service_handle, int instance_id);

int _bt_gatt_server_delete_service(char *sender, int service_handle, int instance_id);

int _bt_gatt_server_send_response(char *sender, bluetooth_gatt_att_data_t *data,
		bluetooth_gatt_server_response_params_t *param);

int _bt_gatt_server_send_indication(char *sender, bluetooth_device_address_t *addr, bluetooth_gatt_att_data_t *data,
		bluetooth_gatt_server_indication_params_t *param);

int _bt_gatt_server_update_attribute_value(char *sender, int instance_id,
		bluetooth_gatt_server_update_value_t *param);

int _bt_get_att_mtu(bluetooth_device_address_t *address,
		unsigned int *mtu);

int _bt_gatt_server_acquire_send_response(char *sender, bluetooth_gatt_server_acquire_response_params_t *param, void *);

#ifdef TIZEN_GATT_CLIENT
char * _bt_gatt_get_default_gatt_client_uuid(void);

int _bt_register_gatt_client_instance(const char *sender,
		bluetooth_device_address_t *address);

gboolean _bt_is_remote_gatt_device_connected(bluetooth_device_address_t *address);


int _bt_connect_le_device(bluetooth_device_address_t *address,
		int auto_connect, int client_id);

int _bt_gatt_get_primary_services(char *address);

int _bt_gatt_get_all_characteristic(bluetooth_gatt_client_svc_prop_info_t *svc);

int _bt_gatt_get_all_characteristic_properties(
		bluetooth_gatt_client_char_prop_info_t *prop);

int _bt_gatt_read_characteristic_value(
		bluetooth_gatt_client_char_prop_info_t *desc);

int _bt_gatt_read_descriptor_value(
		bluetooth_gatt_client_desc_prop_info_t *desc);

int _bt_gatt_write_characteristic_value_by_type(
		bluetooth_gatt_client_char_prop_info_t *chr,
		bluetooth_gatt_att_data_t *data,
		bluetooth_gatt_write_type_e write_type);

int _bt_gatt_acquire_notify(bluetooth_gatt_client_char_prop_info_t *chr, int *fd , int *mtu);

int _bt_gatt_acquire_write(bluetooth_gatt_client_char_prop_info_t *chr, int *fd , int *mtu);

int _bt_gatt_write_descriptor_value_by_type(
		bluetooth_gatt_client_desc_prop_info_t *desc,
		bluetooth_gatt_att_data_t *data,
		bluetooth_gatt_write_type_e write_type);

int _bt_disconnect_le_device(bluetooth_device_address_t *address,
		int client_id);

int _bt_gatt_watch_characteristic(bluetooth_gatt_client_char_prop_info_t *chr,
		int client_id, gboolean is_notify);

int _bt_gatt_watch_service_changed_indication(const char *sender,
		bluetooth_device_address_t *address,
		gboolean is_enabled);

int _bt_unregister_gatt_client_instance(const char *sender,
		int client_id);

#endif
//int _bt_gatt_server_enable_application(int instance_id);
#if 0
int _bt_gatt_server_register(char *sender);

int _bt_gatt_server_unregister(char *sender);

int _bt_gatt_server_update_multi_adv_info(char *name, int instance_id);

int _bt_gatt_server_connect_remote_client(char *sender,
				bluetooth_device_address_t *address, int instance_id);

int _bt_gatt_server_disconnect_remote_client(char *sender,
				bluetooth_device_address_t *address,
				int instance_id);

int _bt_gatt_server_add_service(char *sender, int service_type,
				int num_handles, char *svc_uuid, int instance_id);

int _bt_gatt_server_add_included_service(char *sender, int instance_id,
		int service_handle, int included_svc_handle);

int _bt_gatt_server_add_characteristic(char *sender, char *char_uuid,
				bluetooth_gatt_server_attribute_params_t *param);

int _bt_gatt_server_add_descriptor(char *sender, char *desc_uuid,
				bt_gatt_permission_t *perm, int service_handle, int instance_id);

int _bt_gatt_server_start_service(char *sender, int service_handle, int instance_id);

int _bt_gatt_server_stop_service(char *sender, int service_handle, int instance_id);

int _bt_gatt_server_delete_service(char *sender, int service_handle, int instance_id);

int _bt_gatt_server_send_indication(char *sender, bluetooth_device_address_t *addr, bluetooth_gatt_att_data_t *data,
					bluetooth_gatt_server_indication_params_t *param);

int _bt_gatt_server_send_response(char *sender, bluetooth_gatt_att_data_t *data,
					bluetooth_gatt_server_response_params_t *param);

void _bt_gatt_server_clear_req_info(void);

void _bt_gatt_server_clear_connection_info(void);

bt_product_gatt_permission_t _bt_gatt_convert_attr_permission_to_stack_specific(
		bt_gatt_permission_t perm);

void _bt_check_gatt_server_app_termination(const char *name);

int _bt_gatt_server_get_att_mtu(bluetooth_device_address_t *address, unsigned int *mtu);

#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* BT_SERVICE_GATT_H_ */
